'use strict';

// const builder = require('@finclip/applet-builder')

const getCurTimeString = () => {
  const d = new Date();
  return `${d.getFullYear()}-${d.getMonth() + 1}-${d.getDate()} ${d.getHours()}:${d.getMinutes()}:${d.getSeconds()}`
};

async function buildV2(srcDir, devMode, destPath, tmpPath, option) {
  try {
    if (!option.plugins) option.plugins = [];
    console.log('buildV2 srcDir:', srcDir);
    console.log('buildV2 destPath:', destPath);
    console.log('buildV2 tmpPath:', tmpPath);
    console.log('buildV2 option:', option);

    process.send(['BUILD_DATA', `${getCurTimeString()} 开始加载编译库`]);
    let builder;
    if (option.builderAbsolutePath) {
      builder = require(option.builderAbsolutePath);
    } else {
      builder = require('@finclip/applet-builder');
    }
    process.send(['BUILD_DATA', `${getCurTimeString()} 加载编译库完成`]);

    await builder(devMode, srcDir, destPath, {
      watch: false,
      minify: false,
      obfuscate: false,
      entryFileFolder: tmpPath,
      isFulljs: false,
      onEventCallback: e => {
        e.on('watcher', data => {
          switch (data.code) {
            case 'ERROR':
              process.send(['BUILD_ERROR', data.msg]);
              break
            case 'BUILD_START':
              process.send(['BUILD_START']);
              break
            case 'BUILD_END':
              process.send(['BUILD_SUCCESS', { isWatch: true }]);
              break
            case 'BUILD_RESET':
              process.send(['BUILD_RESET']);
              break
            default:
              break
          }
        });
        e.on('data', data => {
          process.send(['BUILD_DATA', data.toString()]);
        });
      },
      ...option,
    });
  } catch (error) {
    process.send(['BUILD_ERROR', error.message]);
  }
}

process.on('message', msg => {
  switch (msg.type) {
    case 0:
      // buildV1(msg.srcDir, msg.destPath, msg.tmpPath)
      // break
    case 1:
      buildV2(msg.srcDir, msg.devMode, msg.destPath, msg.tmpPath, msg.option);
      break
  }
});

process.on('unhandledRejection', error => {
  process.send(['BUILD_ERROR', error.message]);
});

process.on('uncaughtException', error => {
  process.send(['BUILD_ERROR', error.message]);
});
