
// 此方法由自定义 rollUp 插件注入，用于修复跨 webview clickout 事件不触发的问题
(function () {
  const electron = require('electron')
  const ipcRenderer = electron.ipcRenderer
  document.addEventListener('click', () => { ipcRenderer.send('CLICK_OUT_SIDE') })
  document.addEventListener('touchstart', () => { ipcRenderer.send('CLICK_OUT_SIDE') })
})(require);

// 此方法由自定义 rollUp 插件注入，用于修复 preload 在 electron 中路径依赖问题
(function (module, require) {
  if (/preload|renderer/.test(module.filename)) {
    module.paths.push(require('path').join(module.path, 'app.asar', 'node_modules'));
    console.log(module)
    window.__nodeRequire = require
  }
})(module, require);
'use strict';

var electron = require('electron');
var path = require('path');
var fs = require('fs-extra');
var axios = require('axios');
var uuid = require('uuid');
require('crypto');
require('unzipper');
var mime = require('mime-types');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);
var mime__default = /*#__PURE__*/_interopDefaultLegacy(mime);

const requiredModules = {};
const requireWhenNeed = (module) => {
  return new Proxy(function () {
    if (!requiredModules[module]) requiredModules[module] = window.__nodeRequire(module);
    return requiredModules[module](...arguments)
  }, {
    get: function (target, prop) {
      if (!requiredModules[module]) requiredModules[module] = window.__nodeRequire(module);
      return requiredModules[module][prop]
    }
  })
};

function base64ToArrayBuffer (str) {
  const atobStr = window.atob(str);
  const leng = atobStr.length;
  const arr = new Uint8Array(leng);
  for (let r = 0; r < leng; r++) arr[r] = atobStr.charCodeAt(r);
  return arr.buffer
}

/**
 * 优先使用url地址中的文件后缀，如没有或异常类型，则使用 content-type 生成
 * @param {string} contentType
 * @param {string} [disposition]
 * @param {string} url
 * @returns {string}
 */
 const getFileExt = (url, contentType, disposition) => {
  const fromUrl = () => {
    const urlWithoutParams = url.split('?')[0];
    const isValidUrlExt = mime__default["default"].lookup(urlWithoutParams);
    return isValidUrlExt ? path__default["default"].extname(urlWithoutParams) : ''
  };

  const fromDisposition = () => {
    /**
     * eg:
     * attachment; filename*=UTF-8''app-release.apk; filename="app-release.apk"
     */
    if (!disposition) return ''
    return disposition
      .split(/;\s*/)
      .filter(i => /^filename\s*=/.test(i))
      .map(str => {
        const filename = (str.split('=')[1] || '').trim().replace(/["']$/, '');
        return path__default["default"].extname(filename)
      })
      .pop()
  };

  const fromCtx = () =>{
    const ctxExt = mime__default["default"].extension(contentType);
    return ctxExt ? `.${ctxExt}` : ''
  };
  // console.log(fromUrl() , fromDisposition() , fromCtx());
  return fromUrl() || fromDisposition() || fromCtx()
};

/* eslint-disable no-undef */

axios__default["default"].defaults.withCredentials = true;
const CancelToken = axios__default["default"].CancelToken;
const tasks = {};

const setRequestCookie = async (header = {}, url) => {
  let isSetCookie;
  if (typeof header.Cookie === 'string' || typeof header.cookie === 'string') {
    const cookieStr = header.Cookie || header.cookie;
    delete header.Cookie;
    delete header.cookie;

    await electron.ipcRenderer.invoke('setServiceCookie', {
      url,
      cookieStr,
    });

    isSetCookie = true;
  }
  if (isSetCookie) {
    setTimeout(() => {
      electron.ipcRenderer.invoke('clearServiceCookie', { url });
    }, 500);
  }
};

axios__default["default"].interceptors.request.use(
  async function (config) {
    try {
      await setRequestCookie(config.headers, config.url);
    } catch (error) {
      console.error('setRequestCookie error', error);
    }
    return config
  },
  function (error) {
    // Do something with request error
    return Promise.reject(error)
  }
);

axios__default["default"].interceptors.response.use(
  async function (response) {
    const requestUrl = response.request.responseURL;
    const requestExtraInfo = await electron.ipcRenderer.invoke('getRequestExtraHeader', { url: requestUrl });
    if (requestExtraInfo.setCookie) {
      response.headers['set-cookie'] = requestExtraInfo.setCookie;
    }
    return response
  }
);

// TODO 在这里做域名有效性校验，和小程序配置有关，先默认返回true
const validateUrl = (url) => {
  return true
};

const isFinfile = (path) => {
  return typeof path === 'string' && path.startsWith('finfile://')
};

const resolveFinfilePath = (finfileSrc, cachePath) => {
  return path__default["default"].join(cachePath, finfileSrc.replace('finfile://', ''))
};

const invokeCallback$1 = (statue, event, callbackId, params = {}) => {
  const res = {
    ...params,
    errMsg: `${event}:${statue}${params.errMsg ? ' ' + params.errMsg : ''}`,
  };

  try {
    FinChatJSBridge.invokeCallbackHandler(callbackId, res);
  } catch (error) { }
  try {
    ServiceJSBridge.invokeCallbackHandler(callbackId, res);
  } catch (error) { }
};

const emitCustomEvent = (data) => {
  const { event, params } = data;

  let fcBridgeRes, serviceBridgeRes;
  try {
    fcBridgeRes = FinChatJSBridge.subscribeHandler(event, params, params.webviewId);
  } catch (error) {

  }

  try {
    serviceBridgeRes = ServiceJSBridge.subscribeHandler(event, params, params.webviewId);
  } catch (error) {

  }

  if (fcBridgeRes || serviceBridgeRes) {
    electron.ipcRenderer.send('customEvent-res', {
      returnValue: fcBridgeRes || serviceBridgeRes,
      event,
      params
    });
  }
};

const taskAbort = (event, params, callbackId) => {
  try {
    const { taskId } = params;
    const task = tasks[taskId];
    if (!task) throw new Error('task do not exist')
    if (typeof task.cancel === 'function') {
      task.cancel();
    }
    delete tasks[taskId];
  } catch (error) {
  }
};

// 注意这里会被 webview 的 request 复用
// 修改或移动位置时需要兼容
const request = async (event, params, callbackId) => {
  if (!params.dataType) params.dataType = 'json';
  const { url, data, header, dataType, responseType, method, taskId } = params;

  const handleAxiosRes = axiosResponse => {
    delete tasks[taskId];

    const newHeader = axiosResponse.headers;
    Object.keys(axiosResponse.headers).forEach(key => {
      const upperKey = key
        .split('-')
        .map(text => `${text[0].toUpperCase()}${text.slice(1)}`)
        .join('-');
      // Mark：兼容两种方式，总不会出问题
      newHeader[upperKey] = axiosResponse.headers[key];
      newHeader[key] = axiosResponse.headers[key];
    });

    emitCustomEvent({
      event: 'onTaskHeadersReceived',
      params: { header: newHeader, taskId },
    });

    let response = axiosResponse.request.response;
    if (dataType === 'json') {
      try {
        response = JSON.parse(axiosResponse.request.response);
      } catch (error) {}
    }

    invokeCallback$1('ok', event, callbackId, {
      data: response,
      statusCode: axiosResponse.status,
      cookies:
        (axiosResponse.headers && axiosResponse.headers['set-cookie']) || [],
      profile: {}, // 微信开发工具也没返回这个，先这样返回
      header: newHeader,
    });
  };

  try {
    if (!taskId) throw new Error('invalid taskId')
    if (!validateUrl(url)) ;

    const source = CancelToken.source();
    tasks[taskId] = {
      taskId: taskId,
      type: 'request',
      cancel: () => {
        source.cancel();
        invokeCallback$1('fail', event, callbackId, { errMsg: 'abort' });
      }
    };

    let requestData = data;
    try {
      requestData = JSON.parse(data);
    } catch (error) { }

    // 直接传递 string 给 axios 作为 data 的时候，会在外层外添加一层 "", 使用 buffer 转换一次以规避此问题
    if (typeof requestData === 'string') {
      requestData = Buffer.from(requestData);
    }

    const reqOpts = {
      url,
      method,
      headers: header,
      responseType: responseType === 'arraybuffer' ? 'arraybuffer' : 'text',
      data: requestData,
      cancelToken: source.token
    };

    const res = await axios__default["default"](reqOpts);
    handleAxiosRes(res);
  } catch (error) {
    if (error.response) {
      handleAxiosRes(error.response);
    } else {
      invokeCallback$1('fail', event, callbackId, { errMsg: error.toString() });
      delete tasks[taskId];
    }
  }
};

const uploadFile = async (event, params, callbackId) => {
  const { url, header, name, timeout, formData, filePath, taskId } = params;
  try {
    if (!taskId) throw new Error('invalid taskId')
    if (timeout && typeof timeout !== 'number') throw new Error('timeout must be number')
    if (!validateUrl(url, 'uploadFile')) ;

    const fileCachePath = __ide_project_info.fileCachePath;

    const srcFilePath = isFinfile(filePath) ? resolveFinfilePath(filePath, fileCachePath) : filePath;
    if (!fs__default["default"].existsSync(srcFilePath)) throw new Error('文件不存在')

    const source = CancelToken.source();
    tasks[taskId] = {
      taskId: taskId,
      type: 'uploadFile',
      cancel: () => {
        source.cancel();
        invokeCallback$1('fail', event, callbackId, { errMsg: 'abort' });
      }
    };
    if (timeout && timeout > 0) setTimeout(() => tasks[taskId] && tasks[taskId].cancel('timeout'), timeout);

    const nodeBuffer = fs__default["default"].readFileSync(srcFilePath);
    const arrayBuffer = new Uint8Array(nodeBuffer).buffer;
    // eslint-disable-next-line no-undef
    const fileBlob = new Blob([arrayBuffer]);
    // eslint-disable-next-line no-undef
    const form = new FormData();
    if (typeof formData === 'object') {
      Object.keys(formData).forEach(key => form.append(key, formData[key]));
    }
    form.append(name, fileBlob, path__default["default"].basename(srcFilePath));

    const res = await axios__default["default"].post(url, form, {
      headers: {
        ...header,
        'Content-Type': 'multipart/form-data'
      },
      cancelToken: source.token,
      onUploadProgress: (progressEvent) => {
        const bytesDone = progressEvent.loaded;
        const totalSize = progressEvent.total;
        emitCustomEvent({
          event: 'onTaskProgressUpdate',
          params: {
            taskId,
            progress: totalSize > 0 ? parseInt(bytesDone / totalSize * 100, 10) : NaN,
            totalBytesWritten: bytesDone,
            totalBytesExpectedToWrite: totalSize,
          }
        });
      }
    });
    delete tasks[taskId];

    emitCustomEvent({
      event: 'onTaskHeadersReceived',
      params: { header: res.headers, taskId }
    });

    invokeCallback$1('ok', event, callbackId, {
      data: res.data,
      statusCode: res.status
    });
  } catch (error) {
    invokeCallback$1('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const downloadFile = async (event, params, callbackId) => {
  const { url, header, timeout, filePath, taskId } = params;
  try {
    if (!taskId) throw new Error('invalid taskId')
    if (timeout && typeof timeout !== 'number') throw new Error('timeout must be number')
    if (!validateUrl(url, '')) ;

    const fileCachePath = __ide_project_info.fileCachePath;

    let savePath = '';
    if (filePath) {
      if (fs__default["default"].existsSync(filePath)) throw new Error('filePath 文件已存在')
      fs__default["default"].ensureDirSync(path__default["default"].dirname(filePath));
      savePath = filePath;
    }

    const source = CancelToken.source();
    // const wStream = fs.createWriteStream(savePath)
    tasks[taskId] = {
      taskId: taskId,
      type: 'downloadFile',
      cancel: () => {
        source.cancel();
        // wStream.end(() => {
        //   fs.unlinkSync(savePath)
        // })
        invokeCallback$1('fail', event, callbackId, { errMsg: 'abort' });
      }
    };
    const reqOpts = {
      url,
      method: 'GET',
      headers: header || {},
      cancelToken: source.token,
      responseType: 'blob',
      onDownloadProgress: (progressEvent) => {
        const bytesDone = progressEvent.loaded;
        const totalSize = progressEvent.total;
        emitCustomEvent({
          event: 'onTaskProgressUpdate',
          params: {
            taskId,
            progress: totalSize > 0 ? parseInt(bytesDone / totalSize * 100, 10) : NaN,
            totalBytesWritten: bytesDone,
            totalBytesExpectedToWrite: totalSize,
          }
        });
      }
    };

    if (timeout && timeout > 0) setTimeout(() => tasks[taskId].cancel('timeout'), timeout);

    const res = await axios__default["default"](reqOpts);

    const fileExt = getFileExt(
      url,
      res.headers['content-type'],
      res.headers['content-disposition']
    );
    const fileName = `${uuid.v4()}${fileExt}`;
    if (!filePath) {
      savePath = path__default["default"].join(fileCachePath, 'temp', fileName);
    }

    emitCustomEvent({
      event: 'onTaskHeadersReceived',
      params: { header: res.headers, taskId }
    });

    // eslint-disable-next-line no-undef
    const reader = new FileReader();

    function onLoadEnd(e) {
      reader.removeEventListener('loadend', onLoadEnd, false);
      if (e.error) throw e.error

      const buffer = Buffer.from(reader.result);
      fs__default["default"].writeFileSync(savePath, buffer);

      const returnValue = {
        statusCode: res.status,
        cookies: (res.headers && res.headers['set-cookie']) || [],
        header: res.headers,
        profile: {}, // 微信开发工具也没返回这个，先这样返回
      };
      if (filePath) {
        returnValue.filePath = savePath;
      } else {
        returnValue.tempFilePath = `finfile://temp/${fileName}`;
      }

      delete tasks[taskId];
      invokeCallback$1('ok', event, callbackId, returnValue);
    }
    reader.addEventListener('loadend', onLoadEnd, false);
    reader.readAsArrayBuffer(res.data);
  } catch (error) {
    invokeCallback$1('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const MAX_WEBSOCKET_NUM = 5;
const socketTaskMap = {};

const createSocketTask = async (event, params, callbackId) => {
  const { socketId, url, protocols, timeout } = params;
  try {
    if (Object.keys(socketTaskMap).keys > MAX_WEBSOCKET_NUM) {
      throw new Error(`不能同时连接超过${MAX_WEBSOCKET_NUM}个socket`)
    }

    // https://github.com/websockets/ws/blob/master/doc/ws.md#ready-state-constants
    let readyState = 0;

    // eslint-disable-next-line no-undef
    const websocket = new WebSocket(url, protocols || []);
    socketTaskMap[socketId] = websocket;

    websocket.onopen = function (evt) {
      readyState = 1;
      emitCustomEvent({
        event: 'onSocketOpen',
        params: { socketId, ...evt }
      });
      invokeCallback$1('ok', event, callbackId, evt);
    };
    websocket.onclose = function (evt) {
      readyState = 3;
      emitCustomEvent({
        event: 'onSocketClose',
        params: { socketId, ...evt }
      });
      delete socketTaskMap[socketId];
    };
    websocket.onmessage = async function (evt) {
      let data = evt.data;
      if (evt.data instanceof Blob) {
        data = await evt.data.arrayBuffer();
      }
      emitCustomEvent({
        event: 'onSocketMessage',
        params: { data, socketId }
      });
    };
    websocket.onerror = function (evt) {
      emitCustomEvent({
        event: 'onSocketError',
        params: { socketId, ...evt }
      });
    };

    if (timeout && timeout > 0) {
      setTimeout(() => {
        if (!readyState) {
          websocket.close(1005, 'timeout');
          invokeCallback$1('fail', event, callbackId, { errMsg: 'timeout' });
        }
      }, timeout);
    }
  } catch (error) {
    emitCustomEvent({
      event: 'onSocketError',
      params: { socketId, errMsg: error.toString() }
    });
    invokeCallback$1('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const closeSocket = async (event, params, callbackId) => {
  const { socketId, code, reason } = params;
  try {
    socketTaskMap[socketId].close(code, reason);
    invokeCallback$1('ok', event, callbackId, {});
  } catch (error) {
    emitCustomEvent({
      event: 'onSocketError',
      params: { socketId, errMsg: error.toString() }
    });
    invokeCallback$1('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const sendSocketMessage = async (event, params, callbackId) => {
  let { socketId, data, isBase64 } = params;
  try {
    if (isBase64) {
      data = base64ToArrayBuffer(data);
    }
    socketTaskMap[socketId].send(data);
    invokeCallback$1('ok', event, callbackId, {});
  } catch (error) {
    emitCustomEvent({
      event: 'onSocketError',
      params: { socketId, errMsg: error.toString() }
    });
    invokeCallback$1('fail', event, callbackId, { errMsg: error.toString() });
  }
};

var serviceApi = {
  request,
  taskAbort,
  uploadFile,
  downloadFile,
  createSocketTask,
  closeSocket,
  sendSocketMessage
};

const assembleRes = (statue, event, callbackId, params = {}) => {
  let res;
  if (typeof params === 'object') {
    res = {
      ...params,
      errMsg: `${event}:${statue}${params.errMsg ? ' ' + params.errMsg : ''}`,
    };
  }
  if (typeof params === 'string' || params === 'number') {
    res = {
      errMsg: `${event}:${statue}${params.errMsg ? ' ' + params : ''}`,
    };
  }
  return res
};

const invokeCallback = (statue, event, callbackId, params = {}) => {
  const res = assembleRes(statue, event, callbackId, params = {});
  try {
    FinChatJSBridge.invokeCallbackHandler(callbackId, res);
  } catch (error) { }
  try {
    ServiceJSBridge.invokeCallbackHandler(callbackId, res);
  } catch (error) { }
};

class Callback {
  constructor(event, params, callbackId) {
    this.event = event;
    this.callbackId = callbackId;
    this.params = params;
  }
  onSuccess(res) {
    invokeCallback('ok', this.event, this.callbackId, res);
  }
  onFail(res) {
    invokeCallback('fail', this.event, this.callbackId, res);
  }
  syncRes(res) {
    return assembleRes('ok', this.event, this.callbackId, res)
  }
}


const extApis = {};
const registedExtModules = {};
window.__registedExtModules = registedExtModules;
class ApiContext {
  constructor(modulePath) {
    this.modulePath = modulePath;
    registedExtModules[modulePath] = [];
  }

  onSuccess(callback, res) {
    console.log('onSuccess', callback, res);
    callback.onSuccess(res);
  }

  onFail(callback, res) {
    callback.onFail(res);
  }

  registerApi(apiName, execFunc) {
    const funcWrap = (event, params, callbackId) => {
      const callback = new Callback(event, params, callbackId);
      execFunc.call(this, params, callback);
    };
    wx.loadExtApi({ name: apiName, overwrite: true });
    extApis[apiName] = funcWrap.bind(this);

    registedExtModules[this.modulePath].push({
      apiName,
      sync: false
    });
  }

  registerSyncApi(apiName, execFunc) {
    const funcWrap = (event, params, callbackId) => {
      const callback = new Callback(event, params, callbackId);
      return callback.syncRes(execFunc.call(this, params, callback))
    };
    wx.loadExtApi({ name: apiName, sync: true, overwrite: true });
    extApis[apiName] = funcWrap.bind(this);

    registedExtModules[this.modulePath].push({
      apiName,
      sync: true
    });
  }
}

function registerExtApiModule(modulePath) {
  const userModel = __nodeRequire(modulePath);
  const context = new ApiContext(modulePath);
  userModel(context);
}

electron.ipcRenderer.on('registerExtApiModule', (e, data) => {
  try {
    registerExtApiModule(data);
    electron.ipcRenderer.send('extApiRegisted', { success: true, modulePath: data });
  } catch (error) {
    console.log(error);
    electron.ipcRenderer.send('extApiRegisted', { success: false, errMsg: error.toString() , modulePath: data });
  }
});

window.assert = requireWhenNeed('assert');
window.expect = requireWhenNeed('expect');

let DOMContentLoaded = false;
window.isIDE = true;
window.isSupportFinclipWebsocket = true;
window.__fcjs_user_data_path = 'finfile://usr';

window.__ide_sdk_id = new URL(location.href).searchParams.get('sdkId');
if (window.__ide_sdk_id) {
  const projectInfo = electron.ipcRenderer.sendSync(`getProjectInfo-${window.__ide_sdk_id}`);
  window.__ide_project_info = projectInfo;
  if (projectInfo && projectInfo.deviceInfo && projectInfo.deviceInfo.dpr) {
    window.devicePixelRatio = projectInfo.deviceInfo.dpr;
  }
}

// sdk同步调用的方法在这里注册
const syncEvents = {
  bindUDPPort: true,
  getBatteryInfoSync: true,
  getSystemInfoSync: true,

  setStorageSync: true,
  removeStorageSync: true,
  getStorageSync: true,
  getStorageInfoSync: true,
  clearStorageSync: true,
  getMenuButtonBoundingClientRect: true,
  canvasMethod: true,
  rtcPeerConnection: true,
  mediaDevices: true,
  mediaRecorder: true,
  mediaStream: true
};

const storageApis = {
  setStorageSync: true,
  setStorage: true,
  removeStorageSync: true,
  removeStorage: true,
  clearStorage: true,
  clearStorageSync: true
};

const mockInvokeMap = {};

const syncStorage = (event, params) => { 
  try {
    const { key, data } = params;
    if (event === 'setStorageSync' || event === 'setStorage') {
      const value = JSON.parse(data).data;
      localStorage.setItem(key, value);
    }
    if (event === 'removeStorageSync' || event === 'removeStorage') {
      localStorage.removeItem(key);
    }
    if (event === 'clearStorageSync' || event === 'clearStorage') {
      localStorage.clear();
    }
  } catch (error) {
    console.info('syncStorage error', error);
  }
};

const syncStorageAll = () => {
  window.localStorage.clear();
  const info = electron.ipcRenderer.sendSync('invoke', {
    event: 'getStorageInfoSync',
    params: {},
    callbackId: 0
  });
  info.keys.forEach(key => {
    const res = electron.ipcRenderer.sendSync('invoke', {
      event: 'getStorageSync',
      params: { key },
      callbackId: 0
    });
    try {
      const data = JSON.parse(res.data);
      window.localStorage.setItem(key, JSON.stringify(data.data || data));
    } catch (error) {}
  });
};


window.FinChatJSCore = {
  invokeHandler: function (event, paramsString, callbackId) {
    if (mockInvokeMap[event]) {
      const mockIns = mockInvokeMap[event];
      const res = mockIns.mockCallback(event, paramsString, callbackId);
      ServiceJSBridge.invokeCallbackHandler(callbackId, res);
      if (typeof mockIns.fn === 'function') {
        mockIns.fn();
      }
      if (mockIns.autoCancelMock) {
        delete mockInvokeMap[event];
      }
      return
    }

    const params = JSON.parse(paramsString);

    if (extApis[event]) {
      return extApis[event](event, params, callbackId)
    }

    if (storageApis[event]) {
      syncStorage(event, params);
    }

    if (serviceApi[event]) {
      // 直接在service本地处理
      return serviceApi[event].call(
        this,
        event,
        params,
        callbackId
      )
    }

    // 同步方法直接return
    if (syncEvents[event] || event.endsWith('Sync')) {
      if (event === 'getStorageInfoSync' && !DOMContentLoaded) {
        return {
          errMsg: `${event}: ok`
        }
      }
      return electron.ipcRenderer.sendSync('invoke', {
        event,
        params: params,
        callbackId
      })
    }
    electron.ipcRenderer.send('invoke', {
      event,
      params: params,
      callbackId
    });
  },
  publishHandler: function (event, paramsString, webviewIds) {
    electron.ipcRenderer.send('publish', {
      event,
      params: JSON.parse(paramsString),
      webviewIds
    });
  },
  webInvokeHandler: function (event, paramsString, callbackId) {
    electron.ipcRenderer.send('webInvoke', {
      event,
      params: JSON.parse(paramsString),
      callbackId
    });
  },
  webPublishHandler: function (event, paramsString, webviewIds) {
    electron.ipcRenderer.send(`sdkEvent-${window.__ide_sdk_id}`, {
      event: 'webPublish',
      payload: {
        event,
        params: JSON.parse(paramsString),
        webviewIds
    }});
  }
};

electron.ipcRenderer.on('onAppRoute', (e, data) => {
  ServiceJSBridge.subscribeHandler('onAppRoute', data, data.webviewId);
});

electron.ipcRenderer.on('onServiceReadyDone', (e, data) => {
  ServiceJSBridge.subscribeHandler('onServiceReadyDone', data, data.webviewId);
  syncStorageAll();
});

electron.ipcRenderer.on('syncStorageAll', (e, data) => {
  syncStorageAll();
});

electron.ipcRenderer.on('registerCustomSyncApis', (e, customSyncApiObj) => {
  Object.keys(customSyncApiObj).forEach(key => {
    syncEvents[key] = true;
  });
});

electron.ipcRenderer.on('customEvent', (e, data) => {
  const { event, params, webviewId } = data;

  let fcBridgeRes, serviceBridgeRes;
  try {
    fcBridgeRes = FinChatJSBridge.subscribeHandler(event, params, webviewId || params.webviewId);
  } catch (error) {

  }

  try {
    serviceBridgeRes = ServiceJSBridge.subscribeHandler(event, params, webviewId || params.webviewId);
  } catch (error) {

  }

  if (fcBridgeRes || serviceBridgeRes) {
    let res = fcBridgeRes || serviceBridgeRes;
    if (typeof res === 'function') res = {};
    electron.ipcRenderer.send('customEvent-res', {
      returnValue: res,
      event,
      params
    });
  }
});

electron.ipcRenderer.on('canvasMethod', (e, data) => {
  let fcBridgeRes = null;
  try {
    fcBridgeRes = FinChatJSBridge.subscribeHandler('custom_event_canvasMethod', data);
  } catch (error) {

  }
  electron.ipcRenderer.send(`canvasMethod-${data.uuid}`, {
    data: fcBridgeRes
  });
});

electron.ipcRenderer.on('getSelectedTextRange', (e, data) => {
  const { uuid } = data;
  const activeElement = document.activeElement;
  if (activeElement && ['INPUT', 'TEXTAREA'].includes(activeElement.tagName)) {
    electron.ipcRenderer.send(`getSelectedTextRange-${uuid}`, {
      start: activeElement.selectionStart,
      end: activeElement.selectionEnd
    });
  } else {
    electron.ipcRenderer.send(`getSelectedTextRange-${uuid}`);
  }
});

electron.ipcRenderer.on('invokeCallback', (e, data) => {
  const { callbackId, params } = data;
  try {
    FinChatJSBridge.invokeCallbackHandler(callbackId, params);
  } catch (error) {

  }

  try {
    ServiceJSBridge.invokeCallbackHandler(callbackId, params);
  } catch (error) {

  }
});

electron.ipcRenderer.on('webInvokeCallback', (e, data) => {
  const { callbackId, params } = data;
  try {
    FinChatJSBridge.webInvokeCallbackHandler(callbackId, params);
  } catch (error) {
    console.log(error);
  }
});

document.addEventListener('DOMContentLoaded', () => {
  document.querySelector('html').classList.add('isIDE');
  DOMContentLoaded = true;
});

// 禁止 node 变量暴露到 window 中
const bannedList = ['process'];

bannedList.forEach(key => {
  window[key] = undefined;
});

async function waitFor(selector, option) {
  if (!option.timeout) {
    option.timeout = 120000;
  }
  return new Promise((resolve, reject) => {
    const timer = setTimeout(async () => {
      if (!option.cancel) {
        delete option.timeout;
        let res = document.querySelector(selector);
        if (res) {
          resolve(res);
        } else {
          res = await waitFor(selector, option);
          resolve(res);
        }
      }
    }, 100);
    const timeout = option.timeout;
    if (timeout) {
      setTimeout(() => {
        clearTimeout(timer);
        option.cancel = true;
        reject(new Error(`waitFor selector ${selector} timeout ${timeout} ms`));
      }, timeout);
    }
  })
}
window.waitFor = waitFor;

window.mockInvoke = function (
  mockEvent,
  mockCallback,
  fn,
  autoCancelMock = true
) {
  mockInvokeMap[mockEvent] = {
    mockEvent,
    mockCallback,
    fn,
    autoCancelMock,
  };
};

window.execJavaScriptInView = (path, code) => {
  return new Promise((resolve, reject) => {
    const channel = Date.now();
    electron.ipcRenderer.send('exec_javaScript_in_view', { path, code, channel });
    electron.ipcRenderer.once(
      `exec_javaScript_in_view_response_${channel}`,
      (e, data) => {
        const { error, res } = data;
        if (error) {
          reject(error);
          return
        }
        resolve(res);
      }
    );
  })
};

window.runViewHook = (path, fn = () => { }) => {
  return new Promise((resolve, reject) => {
    const channel = Date.now();
    if (typeof fn === 'function') {
      electron.ipcRenderer.send('run_view_hook', {
        path,
        fn: typeof fn === 'function' ? fn.toString() : fn,
        channel
      });
    }
    electron.ipcRenderer.once(`run_view_hook_response_${channel}`, (e, data) => {
      const { error, res } = data;
      if (error) {
        reject(error);
        return
      }
      resolve(res);
    });
  })
};

window.mockSubscribe = (event, params) => {
  let fcBridgeRes, serviceBridgeRes;
  try {
    fcBridgeRes = FinChatJSBridge.subscribeHandler(
      event,
      params,
      params.webviewId
    );
  } catch (error) { }

  try {
    serviceBridgeRes = ServiceJSBridge.subscribeHandler(
      event,
      params,
      params.webviewId
    );
  } catch (error) { }

  if (fcBridgeRes || serviceBridgeRes) {
    electron.ipcRenderer.send('customEvent-res', {
      returnValue: fcBridgeRes || serviceBridgeRes,
      event,
      params,
    });
  }
};

let viewLoadedCb = null;
window.onViewLoaded = (cb) => {
  viewLoadedCb = cb;
};

electron.ipcRenderer.on('onViewLoaded', (e, data) => {
  if (typeof viewLoadedCb === 'function') {
    viewLoadedCb(data);
  }
  ServiceJSBridge.subscribeHandler('DOMContentLoaded', data.params, data.webviewId);
});
