'use strict';

var electron = require('electron');
var fs = require('fs-extra');
var path = require('path');
var chokidar = require('chokidar');
var emitter = require('events');
var express = require('express');
var http = require('http');
var portfinder = require('portfinder');
var fs$1 = require('fs');
var uuid = require('uuid');
var crypto = require('crypto');
var unzipper = require('unzipper');
var mime = require('mime-types');
var log4js = require('log4js');
var buffer = require('buffer');
var dgram = require('dgram');
var net = require('net');
var ip = require('ip');
var axios = require('axios');
var ChildProcess = require('child_process');
var _fetch = require('node-fetch');
var AbortController = require('abort-controller');
var debounce = require('lodash.debounce');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var chokidar__default = /*#__PURE__*/_interopDefaultLegacy(chokidar);
var emitter__default = /*#__PURE__*/_interopDefaultLegacy(emitter);
var express__default = /*#__PURE__*/_interopDefaultLegacy(express);
var http__default = /*#__PURE__*/_interopDefaultLegacy(http);
var portfinder__default = /*#__PURE__*/_interopDefaultLegacy(portfinder);
var fs__default$1 = /*#__PURE__*/_interopDefaultLegacy(fs$1);
var crypto__default = /*#__PURE__*/_interopDefaultLegacy(crypto);
var unzipper__default = /*#__PURE__*/_interopDefaultLegacy(unzipper);
var mime__default = /*#__PURE__*/_interopDefaultLegacy(mime);
var dgram__default = /*#__PURE__*/_interopDefaultLegacy(dgram);
var net__default = /*#__PURE__*/_interopDefaultLegacy(net);
var ip__default = /*#__PURE__*/_interopDefaultLegacy(ip);
var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);
var ChildProcess__default = /*#__PURE__*/_interopDefaultLegacy(ChildProcess);
var _fetch__default = /*#__PURE__*/_interopDefaultLegacy(_fetch);
var AbortController__default = /*#__PURE__*/_interopDefaultLegacy(AbortController);
var debounce__default = /*#__PURE__*/_interopDefaultLegacy(debounce);

const generateFileName = (srcpath) => {
  return `${uuid.v4()}${path__default["default"].extname(srcpath)}`
};

const isFinfile = (path) => {
  return typeof path === 'string' && path.startsWith('finfile://')
};

const resolveFinfilePath = (finfileSrc, cachePath) => {
  return path__default["default"].join(cachePath, finfileSrc.replace('finfile://', ''))
};

const restoreFinfilePath = (absPath = '', cachePath) => {
  const filePath = absPath.replace(cachePath, '');
  return `finfile://${filePath.replace(/^\//, '')}`
};

const saveFileToTempPath = (srcpath, cachePath) => {
  try {
    fs__default["default"].ensureDirSync(path__default["default"].join(cachePath, 'temp'));
    const fileReName = generateFileName(srcpath);
    const destPath = path__default["default"].join(cachePath, 'temp', fileReName);
    fs__default["default"].copyFileSync(srcpath, destPath);
    const stats = fs__default["default"].statSync(destPath);
    return { path: `finfile://temp/${fileReName}`, size: stats.size }
  } catch (error) {
    console.error(error);
    return null
  }
};

const getDigest = async (srcpath, algorithm = 'md5') => {
  try {
    const hash = crypto__default["default"].createHash(algorithm);
    const res = await new Promise((resolve, reject) => {
      fs__default["default"].createReadStream(srcpath)
        .on('data', data => hash.update(data))
        .on('end', () => resolve(hash.digest('hex')));
    });
    return res
  } catch (error) {
    console.error(error);
    return null
  }
};

const isFileExist = function (dir) {
  try {
    fs__default["default"].statSync(dir);
    return true
  } catch (e) {
    // console.log('DEBUG: isFileExist -> e', e)
    return false
  }
};

const getProjectRoot = function () {
  const projectRoot = path__default["default"].join(electron.app.getPath('userData'), `./preview/miniapp/${global.previewSDK.id || global.previewSDK.name}/app`);
  return projectRoot
};

// 把指定路径文件拷贝到目标目录下
async function mergeDir(file, target) {
  const files = fs__default["default"].readdirSync(file);
  for (let i = 0; i < files.length; i++) {
    const item = files[i];
    const fullPath = file + '/' + item;
    const targetFullPath = target + '/' + item;
    const stats = fs__default["default"].statSync(fullPath);
    if (stats.isDirectory()) {
      if (isFileExist(targetFullPath)) {
        await mergeDir(fullPath, targetFullPath);
      } else {
        await fs__default["default"].copy(fullPath, targetFullPath);
      }
    } else {
      await fs__default["default"].copy(fullPath, targetFullPath);
    }
  }
}

function readFileSync$1 (src) {
  return fs__default["default"].readFileSync(src).toString()
}

async function unzipFile (src, target) {
  try {
    console.log('unzipFile', src, target);
    const unzipStream = fs__default["default"].createReadStream(src);
    const extractStream = unzipper__default["default"].Extract({ path: target });
    unzipStream.pipe(extractStream);
    return new Promise((resolve, reject) => {
      unzipStream.on('close', () => {
        console.log('unzipFile done', src, target);
        resolve(true);
      });
      extractStream.on('error', (error) => reject(error));
      unzipStream.on('error', (error) => {
        reject(error);
      });
    })
  } catch (error) {
    console.error(error);
  }
}

/**
 * 优先使用url地址中的文件后缀，如没有或异常类型，则使用 content-type 生成
 * @param {string} contentType
 * @param {string} [disposition]
 * @param {string} url
 * @returns {string}
 */
 const getFileExt = (url, contentType, disposition) => {
  const fromUrl = () => {
    const urlWithoutParams = url.split('?')[0];
    const isValidUrlExt = mime__default["default"].lookup(urlWithoutParams);
    return isValidUrlExt ? path__default["default"].extname(urlWithoutParams) : ''
  };

  const fromDisposition = () => {
    /**
     * eg:
     * attachment; filename*=UTF-8''app-release.apk; filename="app-release.apk"
     */
    if (!disposition) return ''
    return disposition
      .split(/;\s*/)
      .filter(i => /^filename\s*=/.test(i))
      .map(str => {
        const filename = (str.split('=')[1] || '').trim().replace(/["']$/, '');
        return path__default["default"].extname(filename)
      })
      .pop()
  };

  const fromCtx = () =>{
    const ctxExt = mime__default["default"].extension(contentType);
    return ctxExt ? `.${ctxExt}` : ''
  };
  // console.log(fromUrl() , fromDisposition() , fromCtx());
  return fromUrl() || fromDisposition() || fromCtx()
};

emitter__default["default"].setMaxListeners(0);
const logPath = path__default["default"].join(electron.app.getPath('userData'), 'log', 'app.log');
const logConfig = {
  appenders: {
    default: {
      type: 'file',
      filename: logPath,
      pattern: 'yyyy-MM-dd',
      compress: false,
      keepFileExt: true,
      maxLogSize: 10000000,
    },
    console: {
      type: 'console',
    },
  },
  categories: {
    default: { appenders: ['default'], level: 'all' }
  },
  replaceConsole: true,
};
log4js.configure(logConfig);

const setLogger = (name, filePath) => {
  logConfig.appenders[name] = {
    type: 'file',
    filename: filePath,
    pattern: 'yyyy-MM-dd',
    compress: false,
    keepFileExt: true,
    maxLogSize: 10000000,
  };
  logConfig.categories[name] = {
    appenders: [name],
    level: 'all',
  };
  log4js.configure(logConfig);
};

const createEventLogger = ({ app, type }) => {
  try {
    setLogger(type, path__default["default"].join(app.userDataPath, 'log', `${app.id || app.name}`, `${type}.log`));
    return log4js.getLogger(type)
  } catch (error) {
    console.error(error);
    return {}
  }
};

setLogger('builder', path__default["default"].join(electron.app.getPath('userData'), 'log', 'build.log'));
const buildLogger = log4js.getLogger('builder');

class EventManager {
  /**
   * @param {import('#/index').default} app 
   */
  constructor(app, eventId, id, type) {
    this.app = app;
    this.id = id;
    this.type = type;
    this.eventId = eventId;
    this.listeners = [];
    this.registerHandles = [];
    this.webContent = null;

    const handleRegist = this._handleRegist.bind(this);
    const handleLog = this._handleLog.bind(this);

    electron.ipcMain.handleOnce(`regist_${this.eventId}`, handleRegist);
    electron.ipcMain.on('webview_log', handleLog);

    this.listeners.push({ event: `regist_${this.eventId}`, tempFunc: handleRegist });
    this.listeners.push({ event: 'webview_log', tempFunc: handleLog });
  }

  _handleRegist(e, data) {
    this.logger = createEventLogger({ app: this.app, type: this.type });
    const { id } = data;
    this.id = id;

    this.webContent = electron.webContents.fromId(this.id);

    if (this.webContent) {
      this.webContent.on;
    }

    this.registerHandles.forEach(cb => {
      if (typeof cb === 'function') {
        cb(this.id, this.eventId);
      }
    });

    this.registerHandles = [];
    return id
  }

  _handleLog(ipcEvent, data) {
    if (ipcEvent.sender.id === this.id) {
      this.logger && this.logger.info(`log ${data.type} ${JSON.stringify(data)}`);
    }
  }

  destroy() {
    try {
      this.listeners.forEach(item => {
        electron.ipcMain.off(item.event, item.tempFunc);
      });
    } catch (error) {}
  }

  on(event, callback) {
    const tempFunc = (ipcEvent, data) => {
      if (ipcEvent.sender.id === this.id) {
        callback(data, ipcEvent);
        this.logger &&
          this.logger.info(`on ${event}, ${JSON.stringify(data)}`);
      }
    };
    this.listeners.push({ event, tempFunc });
    electron.ipcMain.on(event, tempFunc);
  }

  onRegister(callback) {
    if (this.id) {
      if (typeof callback === 'function') {
        callback(this.id, this.eventId);
      }
    } else {
      this.registerHandles.push(callback);
    }
  }

  once(event, callback) {
    const tempFunc = (ipcEvent, data) => {
      if (ipcEvent.sender.id === this.id) {
        callback(data, ipcEvent);
        this.logger && this.logger.info(`once ${event}, ${JSON.stringify(data)}`);
      }
    };
    this.listeners.push({ event, tempFunc });
    electron.ipcMain.once(event, tempFunc);
  }

  send(event, data) {
    this.logger && this.logger.info(`send ${event}, ${JSON.stringify(data)}`);
    this.webContent.send(event, data);
  }
}

class View {
  /**
   * @param {import('#/index').default} app 
   */
  constructor(app, path, query, openType) {
    this.app = app;
    this.webviewId = uuid.v4();
    this.path = path;
    this.query = query;
    this.openType = openType;
    this.isDomLoaded = false;
    this.isDestroyed = false;
    this.domLoadedCallbacks = [];
    this.webviewComponent = false;
    this.create();
  }

  send(event, data) {
    this.event.send(event, data);
  }

  create() {
    this.app._emitSdkEvent('CREATE_WEBVIEW',{
      src: this.path,
      webviewId: this.webviewId,
      type: 'view'
    });
    this.event = new EventManager(this.app, this.webviewId, null, encodeURIComponent(this.path));
    this.event.on('invoke', (data) => {
      this.handleInvoke(data.event, data.params, data.callbackId);
    });
    this.event.on('publish', (data) => {
      this.handlePublish(data.event, data.params);
    });
  }

  destroy() {
    if (this.isDestroyed) return
    this.app._emitSdkEvent('DESTROY_WEBVIEW', {
      webviewId: this.webviewId,
    });
    console.log('view destroyWebview', this.webviewId);
    this.event.destroy();
    if (this.webviewComponent) {
      this.app.webviewSDK.destroyWebviewById(this.webviewComponent);
    }
    this.isDestroyed = true;
  }

  onDomLoaded(callback) {
    if (typeof callback !== 'function') return
    if (this.isDomLoaded) {
      callback();
    } else {
      this.domLoadedCallbacks.push(callback);
    }
  }

  handleInvoke(event, params, callbackId) {
    const data = {
      ...params,
      webviewId: this.webviewId
    };
    switch (event) {
      case 'insertHTMLWebView':
        this.app.webviewSDK.insertHTMLWebView(data);
        this.webviewComponent = data.htmlId;
        break
      case 'updateHTMLWebView':
        this.app.webviewSDK.updateHTMLWebView(data);
        break
      case 'removeHTMLWebView':
        this.app.webviewSDK.removeHTMLWebView(data);
        this.webviewComponent = null;
        break
    }
  }

  handlePublish(event, params) {
    switch (event) {
      case 'custom_event_DOMContentLoaded':
        this.onLoad(this, event, params);
        if (this.isDomLoaded) return
        this.isDomLoaded = true;
        this.app.sendToService('onViewLoaded', {
          eventId: this.event.id,
          webviewId: this.event.eventId,
          path: this.path,
          query: this.query,
          openType: this.openType,
          params
        });
        this.domLoadedCallbacks.forEach(cb => {
          cb();
        });
        this.domLoadedCallbacks = [];

        break
      case 'custom_event_getImageBase64':
        let filePath = '';
        if (isFinfile(params.src)) {
          filePath = resolveFinfilePath(params.src, this.app.fileCachePath);
        } else {
          filePath = path__default["default"].join(this.app.compileOutputPath, params.src);
        }
        if (filePath && isFileExist(filePath)) {
          this.send('customEvent', {
            event,
            params: {
              src: params.src,
              data: `data:image/png;base64,${fs__default$1["default"].readFileSync(filePath, { encoding: 'base64' })}`
            }
          });
        }
        break
      default:
        if (event.startsWith('custom_event_')) {
          this.app.sendToService(
            'customEvent',
            {
              event,
              webviewId: this.webviewId,
              params
            }
          );
        }
        break
    }
  }
}

const getProjectSubpackByName = (previewSDK, name) => {
  const projectRoot = path__default["default"].join(previewSDK.userDataPath, `./preview/miniapp/${previewSDK.id || previewSDK.name}/${name}`);
  return projectRoot
};

/**
 * @typedef {{
 *    openType: 'appLaunch'|'switchTab'|'navigateBack'|'reLaunch'|'redirectTo',
 *    webviewId: number,
 *    path: string,
 *    query?: Record<string,unknown> 
 *    tabIndex?: number
 * }} RouteData
 */

class Route {
  /**
   * @param {import('#/index').default} app 
   */
  constructor(app) {
    this.isReady = false;
    /**
     * @type {View[]}
     */
    this.stack = [];
    /**
     * @type {View[]}
     */
    this.tabs = [];
    this.activeTab = null;
    this.launchView = null;
    this.currentView = null;
    /**
     * @typedef {{routeData:RouteData, load: boolean, ready: boolean}} subpackData
     * @type {Record<string, subpackData>}
     */
    this.subpackageData = {};
    this.app = app;
    this.currentPath = '';
    this.init();
  }

  init() {
    this.app.on('CALL_ROUTE_FUNC', data => {
      switch (data.type) {
        case 'navigateBack':
          this.navigateBack();
          break
        case 'switchTab':
          this.switchTab(data.path, data.tabIndex);
          break
        case 'relaunch':
          this.reLaunch(data.path);
          break
      }
    });

    this.app.on('LOAD_SUBPACKAGE_CONFIG_DONE', data => {
      console.log('LOAD_SUBPACKAGE_CONFIG_DONE');
      this._routeBySubpackage(data);
    });
  }

  reload() {
    this.isReady = false;
    this.stack.forEach(view => {
      view.destroy();
    });
    this.tabs.forEach(view => {
      view.destroy();
    });
    this.stack = [];
    this.tabs = [];
    this.subpackageData = {};
    this.launchView = null;
    this.currentView = null;
  }

  onReady() {
    this.isReady = true;
    const { launchView, currentView } = this;
    if (launchView) {
      const data = {
        openType: launchView.openType,
        webviewId: launchView.webviewId,
        path: launchView.path,
        query: launchView.query,
      };
      this._route(data);
      if (currentView) {
        const currentViewData = {
          openType: currentView.openType,
          webviewId: currentView.webviewId,
          path: currentView.path,
          query: currentView.query,
        };
        this._route(currentViewData);
      }
    }
  }

  /**
   * 
   * @param {RouteData} data 
   */
  _route(data) {
    this.app._emitSdkEvent('ROUTE', { ...data, stackLength: this.stack.length });
    this.app.sendToService('onAppRoute', data);
    this.currentPath = data.path;
  }

  /**
   * 
   * @param {string} url 
   * @returns 
   */
  _parseUrl(url) {
    const [path, search] = url.split('?');
    const query = {};
    for (const [key, value] of new URLSearchParams(search).entries()) {
      query[key] = value;
    }
    return { path, query, param: search }
  }

  /**
   * 分包的路由处理
   * @param {{ subPath: string[]}} params 分包的路径地址
   */
  _routeBySubpackage(params) {
    params.subPath.forEach(p => {
      const data = this.subpackageData[p];
      if (data) {
        if (data.load) {
          this._route(data.routeData);
        } else {
          this.subpackageData[p].ready = true;
        }
      }
    });
  }

  /**
   * 创建页面
   * @param { String } url 页面路径
   * @param {RouteData['openType']} openType 以什么方式创建（switchTab/appLaunch/navigateBack)
   * @param {boolean} [isTab] 是否是tab
   */
  async _createPage(url, openType, isTab = false) {
    const { path, query } = this._parseUrl(url);
    const appConfig = this.app.ftConfig;
    const pagePath = path.slice(0, -5);
    // 启用了分包加载的处理
    if (appConfig && (appConfig.subpackages || appConfig.subPackages)) {
      const subpackage = (
        appConfig.subpackages ||
        appConfig.subPackages ||
        []
      ).find(i => i.pageFullPath.includes(pagePath));
      if (subpackage) {
        const subPath = getProjectSubpackByName(this.app, subpackage.packageName);
        if (subpackage.packageName !== '__APP__' && fs__default$1["default"].existsSync(subPath)) {
          if (!this.subpackageData[pagePath]) {
            // 将该分包模块合并到指定目录下
            // await mergeDir(subPath, this.app.compileOutputPath)
            const config = readFileSync$1(
              subPath + '/' + '__subPackageConfig.json'
            );
            this.app._emitSdkEvent('LOAD_SUBPACKAGE_CONFIG', {
              config: JSON.parse(config),
              subPath: subpackage.pageFullPath,
            });
            await new Promise(resolve => {
              this.app.on('LOAD_SUBPACKAGE_CONFIG_DONE', data => {
                resolve();
              });
            });
            subpackage.pageFullPath.forEach(p => {
              this.subpackageData[p] = {};
            });
          }
        }
      }
    }
    const view = new View(this.app, path, query, openType);
    const { launchView } = this;
    // 更新tabs记录以及route页面栈
    if (isTab) {
      this.tabs.push(view);
      if (view.openType === 'appLaunch' || view.openType === 'reLaunch') {
        this.stack.push(view);
      }
    } else {
      this.stack.push(view);
    }
    const data = {
      openType: view.openType,
      webviewId: view.webviewId,
      path: view.path,
      query: view.query,
    };
    if (this.subpackageData[pagePath]) {
      this.subpackageData[pagePath].routeData = Object.assign({}, data);
    }
    view.onLoad = params => {
      if (isTab && view.openType !== 'appLaunch' && view.openType !== 'reLaunch') {
        // Mark: tab 预加载因此不需要在 onload 的时候跳转
        return
      }
      if (this.isReady) {
        // 如果是分包的页面应该等config加载完再调用
        this.currentView = view;
        this._route(data);
      } else {
        if (view.openType === 'appLaunch' || view.openType === 'reLaunch') {
          this.launchView = data;
        } else if (!launchView) {
          this.launchView = {
            ...data,
            openType: 'appLaunch',
          };
        } else if (view.path !== launchView.path) {
          this.currentView = data;
        }
      }
    };
    view.sendToService = (event, data) => {
      this.app.sendToService(event, data);
    };
  }

  async launch(path, openType, isTab) {
    await this._createPage(path, openType, isTab);
  }

  async preload(path, openType, isTab) {
    await this._createPage(path, openType, isTab);
  }

  checkPathIsExist(url) {
    let { path } = this._parseUrl(url);
    // pages 保存的路径是跟小程序 app.json 一致的
    // url 是基础库传过来的绝对路径且带 .html, 所以得稍作处理
    if (path.startsWith('/')) {
      path = path.slice(1);
    }

    if (path.endsWith('.html')) {
      path = path.replace('.html', '');
    }

    return (this.app.ftConfig.pages || []).includes(path)
  }

  async navigateTo(path, openType) {
    await this._createPage(path, openType);
  }

  // 相关demo: page-container https://wiki.finogeeks.club/download/attachments/161317402/minicode-9.zip?version=1&modificationDate=1630892064100&api=v2
  navigateBack(delta = 1) {
    if (this.stack.length <= delta) {
      delta = this.stack.length - 1;
    }
    // 获取需要移除的页面
    if (delta > 0) {
      const removeItems = [];
      let i = 0;
      while (i < delta) {
        const oldItem = this.stack.pop();
        if (oldItem.opentType === 'appLaunch' || oldItem.opentType === 'reLaunch' || oldItem.openType === 'switchTab') {
          i = delta;
          this.stack.push(oldItem);
        } else {
          removeItems.push(oldItem);
          i++;
        }
      }

      let viewItem = this.stack[this.stack.length - 1];
      if (this.stack.length - 1 === 0 && this.activeTab) {
        const tabItem = this.tabs.find(item => item.path === this.activeTab);
        if (tabItem) viewItem = tabItem;
      }
      const data = {
        openType: 'navigateBack',
        webviewId: viewItem.webviewId,
        path: viewItem.path,
        query: viewItem.query,
      };
      this._route(data);
      this.currentView = viewItem;
      removeItems.forEach(item => {
        item.destroy();
      });
    }
  }

  async redirectTo(path, openType) {
    const viewItem = this.stack.pop();
    const isTab = this.tabs.find(item => item.path === viewItem.path);
    if (viewItem && !isTab) {
      setTimeout(() => {
        viewItem.destroy();
      }, 100);
    }
    await this._createPage(path, openType);
  }

  async switchTab(path, tabIndex) {
    const targetTab = this.tabs.find(item => {
      return item.path === path
    });
    if (targetTab) {
      this.stack.forEach(viewItem => {
        const isTab = this.tabs.find(tabItem => tabItem.path === viewItem.path);
        if (!isTab) viewItem.destroy();
      });
      this.stack = [targetTab];
      this.lastActiveTab = targetTab;
      const data = {
        openType: 'switchTab',
        webviewId: targetTab.webviewId,
        path: targetTab.path,
        tabIndex: tabIndex
      };
      targetTab.onDomLoaded(() => {
        this.currentView = targetTab;
        this._route(data);
      });
    } else {
      await this._createPage(path, 'switchTab', true);
    }
  }

  reLaunch(path, openType) {
    this.stack.forEach(viewItem => {
      viewItem.destroy();
    });
    this.tabs.forEach(viewItem => {
      viewItem.destroy();
    });
    this.stack = [];
    this.tabs = [];

    let reLaunchOpts = {};
    if (path) {
      reLaunchOpts = this._parseUrl(path);
      reLaunchOpts.path = reLaunchOpts.path.replace('.html', '');
    }
    this.app.service.launchInitPages(this.app.ftConfig, reLaunchOpts);
  }

  getView(webviewId) {
    return this.stack.find(viewItem => viewItem.webviewId === webviewId)
  }
}

const formatHeader = (rawHeader) => {
  const retrunValue = {};
  Object.keys(rawHeader).forEach(key => { retrunValue[key] = rawHeader[key].join(','); });
  return retrunValue
};

// TODO 在这里做域名有效性校验，和小程序配置有关，先默认返回true
const validateUrl = (url) => {
  return true
};

const parseQuery = (queryString) => {
  try {
    const query = {};
    const pairs = (
      queryString[0] === '?' ? queryString.substr(1) : queryString
    ).split('&');
    for (let i = 0; i < pairs.length; i++) {
      const pair = pairs[i].split('=');
      query[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1] || '');
    }
    return query
  } catch (error) {
    console.error(error);
    return {}
  }
};

const defaultDeviceInfo = {
  device: 'iPhone 5',
  percent: '100%',
  width: 320,
  height: 568,
  dpr: 2,
};

const TABBAR_HEIGHT = 20;
const NAVIGATIONBAR_HEIGHT = 34;

const genSystemInfo = (previewSDK) => {
  const deviceInfo = previewSDK.deviceInfo || defaultDeviceInfo || {};
  const appletInfo = previewSDK.appStore.data.currentAppletVersion || {};
  const mockSystemInfo = {
    SDKVersion: appletInfo.version,
    batteryLevel: 98,
    benchmarkLevel: 1,
    brand: 'devtools',
    deviceOrientation: 'portrait',
    devicePixelRatio: deviceInfo.dpr,
    fontSizeSetting: 15,
    language: 'zh_CN',
    model: deviceInfo.name || deviceInfo.device,
    pixelRatio: deviceInfo.dpr,
    platform: 'IDE',
    system: deviceInfo.system || 'iOS',
    inFinChat: true,
    safeArea: {
      left: 0,
      right: deviceInfo.width,
      top: TABBAR_HEIGHT,
      bottom: deviceInfo.height,
      width: deviceInfo.width,
      height: deviceInfo.height - TABBAR_HEIGHT
    },
    screenHeight: deviceInfo.height,
    screenWidth: deviceInfo.width,
    windowHeight: deviceInfo.height - (TABBAR_HEIGHT + NAVIGATIONBAR_HEIGHT),
    windowWidth: deviceInfo.width,
    tabBarHeight: 62,
    statusBarHeight: 24,
    navBarHeight: 56,
  };
  return mockSystemInfo
};

const getAppConfig = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {
    theme: {}
  });
};

const getSuperviseInfo = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {
    actualMobile: '',
    appName: 'FinChat小程序',
    appVersion: '0.0.1',
    iccid: '',
    idfv: '',
    imei: '',
    imsi: '',
    innerIp: '10.0.2.15',
    macAddress: '02:15:B2:00:00:00',
    publicIp: '',
    publicPort: '',
    registerMobile: '',
    systemVersion: 'Android 11',
    udid: '39cd40e5bdb865f145d048e597ed5e53',
  });
};

const reportAction = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const getPublicLibVersion = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const getSystemInfo = (event, callbackId, params, ipcEvent, previewSDK) => {
  const systemInfo = genSystemInfo(previewSDK);
  previewSDK.service.invokeCallback('ok', event, callbackId, systemInfo);
};

const getSystemInfoSync = (event, callbackId, params, ipcEvent, previewSDK) => {
  const systemInfo = genSystemInfo(previewSDK);
  previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, systemInfo);
};

const authorize = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const onThemeChange = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
};

const offThemeChange = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
};

const getSetting = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {
    authSetting: {}
  });
};

const basicHandler = {
  getAppConfig,
  getSuperviseInfo,
  reportAction,
  getPublicLibVersion,
  getSystemInfo,
  getSystemInfoSync,
  authorize,
  onThemeChange,
  offThemeChange,
  getSetting
};

const getClipboardData = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {
    data: electron.clipboard.readText()
  });
};

const setClipboardData = (event, callbackId, params, ipcEvent, previewSDK) => {
  if (params && typeof params.data === 'string') {
    electron.clipboard.writeText(params.data);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } else {
    previewSDK.service.invokeCallback('ok', event, callbackId, {
      errMsg: 'data必须为string'
    });
  }
};

const getScreenBrightness = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {
    value: 0.5
  });
};

const setScreenBrightness = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const vibrateShort = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};
const vibrateLong = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const setKeepScreenOn = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const qrCodeData = {
  charSet: 'UTF-8',
  rawData: 'dGhpcyBpcyBkYXRhIGluIFFSQ29kZQ==',
  result: 'this is data in QRCode',
  scanType: 'QR_CODE',
};

const scanCode = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, qrCodeData);
};

electron.ipcMain.on('setQrCodeScanRetrunValue', (event, payload) => {
  qrCodeData.result = payload.value;
  qrCodeData.rawData = Buffer.from(payload.value).toString('base64');
});

let accelerometerIntervalId;
const accelerometerValue = {
  x: 0,
  y: 0,
  z: 0
};

let isShaking = false;
const simulateShake = () => {
  const SIMULATE_INTERVAL = 20;
  const SIMULATE_DURATION = 1000;
  if (isShaking) return
  isShaking = true;

  let time = 0;
  const shakeInterval = setInterval(() => {
    accelerometerValue.x = 3 * Math.sin(time * 500 / Math.PI);
    accelerometerValue.y = accelerometerValue.x;
    accelerometerValue.z = accelerometerValue.x;
    time += SIMULATE_INTERVAL;
  }, SIMULATE_INTERVAL);

  setTimeout(() => {
    clearTimeout(shakeInterval);
    isShaking = false;
    accelerometerValue.x = 0;
    accelerometerValue.y = 0;
    accelerometerValue.z = 0;
  }, SIMULATE_DURATION);
};

electron.ipcMain.on('simulateShake', (event) => { simulateShake(); });

const startAccelerometer = (event, callbackId, params, ipcEvent, previewSDK) => {
  const intervalMap = {
    game: 20,
    ui: 60,
    normal: 200
  };
  const interval = intervalMap[params.interval] || 200;

  if (accelerometerIntervalId) clearInterval(accelerometerIntervalId);
  accelerometerIntervalId = setInterval(() => {
    previewSDK.service && previewSDK.service.send('customEvent', {
      event: 'onAccelerometerChange',
      params: accelerometerValue
    });
  }, interval);
  previewSDK.service.invokeCallback('ok', event, callbackId, { });
};

const enableAccelerometer = (event, callbackId, params, ipcEvent, previewSDK) => {
  const { enable } = params;
  if (enable) {
    if (accelerometerIntervalId) clearInterval(accelerometerIntervalId);
    accelerometerIntervalId = setInterval(() => {
      previewSDK.service && previewSDK.service.send('customEvent', {
        event: 'onAccelerometerChange',
        params: accelerometerValue
      });
    }, 200);
  } else {
    clearInterval(accelerometerIntervalId);
    accelerometerIntervalId = null;
  }
  previewSDK.service.invokeCallback('ok', event, callbackId, { });
};

const stopAccelerometer = (event, callbackId, params, ipcEvent, previewSDK) => {
  clearInterval(accelerometerIntervalId);
  accelerometerIntervalId = null;
  previewSDK.service.invokeCallback('ok', event, callbackId, { });
};

const deviceHandler = {
  getClipboardData,
  setClipboardData,
  getScreenBrightness,
  setScreenBrightness,
  vibrateShort,
  vibrateLong,
  setKeepScreenOn,
  scanCode,
  startAccelerometer,
  stopAccelerometer,
  enableAccelerometer
};

/**
 * @typedef {{ mode: number, size: number, lastAccessedTime: number, lastModifiedTime: number }} statObject
 *
 * @param {fs.Stats} stat
 * @returns {statObject}
 */
const fsStatToFinclipStat = (stat) => {
  return {
    size: stat.size,
    mode: stat.mode,
    lastAccessedTime: Math.floor(stat.atimeMs / 1000),
    lastModifiedTime: Math.floor(stat.mtimeMs / 1000),
  }
};
function parseNumber(num, defaultValue) {
  if (isNaN(num) || num === "") {
    return defaultValue
  } else {
    return Number(num)
  }
}

/**
 *
 * @param {string} filePath
 * @param {("store"|"usr"|"temp")[]} [permissions]
 * @returns string|null
 */
const checkFilePermission = (filePath, permissions = ['store', 'usr', 'temp']) => {
  const valid = permissions.some((dir) => filePath.startsWith(`finfile://${dir}/`));
  if (!valid) throw new Error(`permission denied, open ${filePath}`)
};

const isDirectory = (filePath) => fs__default["default"].statSync(filePath).isDirectory();

const removeSavedFile = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { filePath } = params;
    if (!isFinfile(filePath)) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'filePath 参数错误' });
    }
    checkFilePermission(filePath, ['store']);
    const fileAbsolutePath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    if (isDirectory(fileAbsolutePath)) {
      throw new Error(`${filePath} is directory`)
    }
    fs__default["default"].removeSync(fileAbsolutePath);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') });
  }
};

const openDocument$1 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { filePath } = params;
    if (!isFinfile(filePath)) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'filePath 参数错误' });
    }
    const fileAbsolutePath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isFileExsit = fs__default["default"].existsSync(fileAbsolutePath);
    if (!isFileExsit) previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: '文件不存在' });
    electron.shell.openPath(fileAbsolutePath);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') });
  }
};

const getFileInfo = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { filePath, digestAlgorithm = 'md5' } = params;
    if (!isFinfile(filePath)) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'tempFilePath 参数错误' });
    }
    if (typeof digestAlgorithm === 'string' && !(digestAlgorithm.toLocaleLowerCase() === 'md5' || digestAlgorithm.toLocaleLowerCase() === 'sha1')) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'digestAlgorithm 参数错误' });
    }
    const fileAbsolutePath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isFileExsit = fs__default["default"].existsSync(fileAbsolutePath);
    if (!isFileExsit) previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: '文件不存在' });
    const isDirectory = fs__default["default"].statSync(fileAbsolutePath).isDirectory();
    if (isDirectory) {
      throw new Error(`${filePath} is directory`)
    }
    const res = {};
    res.digest = await getDigest(fileAbsolutePath, digestAlgorithm);
    res.size = fs__default["default"].statSync(fileAbsolutePath).size;
    previewSDK.service.invokeCallback('ok', event, callbackId, res);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') });
  }
};

const getSavedFileList = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const storePath = path__default["default"].join(previewSDK.fileCachePath, 'store');
    const res = [];
    fs__default["default"].readdir(storePath, (err, files = []) => {
      if (err) previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: err.toString() });
      files.forEach(file => {
        const stats = fs__default["default"].statSync(path__default["default"].join(storePath, file));
        stats && res.push({
          filePath: `finfile://store/${file}`,
          size: stats.size,
          createTime: parseInt(stats.birthtime.getTime() / 1000)
        });
      });
      previewSDK.service.invokeCallback('ok', event, callbackId, { fileList: res });
    });
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') });
  }
};

const getSavedFileInfo = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { filePath } = params;
    if (!isFinfile(filePath)) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'tempFilePath 参数错误' });
    }
    const fileAbsolutePath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isFileExsit = fs__default["default"].existsSync(fileAbsolutePath);
    if (!isFileExsit) previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: '文件不存在' });
    const res = {};
    const stats = fs__default["default"].statSync(fileAbsolutePath);
    res.createTime = parseInt(stats.birthtime.getTime() / 1000);
    res.size = fs__default["default"].statSync(fileAbsolutePath).size;
    previewSDK.service.invokeCallback('ok', event, callbackId, res);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') });
  }
};

const fileToBase64 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  const { url } = params;
  if (!url.startsWith('finfile://')) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'url 参数错误' });
    return
  }
  const pathname = path__default["default"].join(previewSDK.fileCachePath, decodeURI(url.replace('finfile://', '')));
  const isFileExsit = fs__default["default"].existsSync(pathname);
  if (!isFileExsit) return previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: '文件不存在' })
  const buffer = fs__default["default"].readFileSync(pathname);
  previewSDK.service.invokeCallback('ok', event, callbackId, {
    data: buffer.toString('base64')
  });
};

const base64ToTempFilePath = async (event, callbackId, params, ipcEvent, previewSDK) => {
  const uuid$1 = uuid.v4();
  previewSDK.homeEventManager.once(`base64ToTempFilePath-${uuid$1}`, (res) => {
    const fileName = `${uuid$1}.png`;
    const filePath = path__default["default"].join(previewSDK.fileCachePath, 'temp', fileName);
    fs__default["default"].ensureDirSync(path__default["default"].dirname(filePath));
    fs__default["default"].writeFileSync(filePath, buffer.Buffer.from(res.data, 'base64'));
    fs__default["default"].writeFileSync(filePath.replace('.png', '.json'), res.data);
    previewSDK.service.invokeCallback('ok', event, callbackId, {
      tempFilePath: `finfile://temp/${fileName}`
    });
  });
  previewSDK.sendToHome('base64ToTempFilePath', {
    ...params,
    uuid: uuid$1
  });
};

// file system manager api
const access = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { path: filePath = '' } = params;

    // temp 里 file 可以读，但不能写
    if (!/^finfile:\/\/(temp|usr|store)/.test(filePath)) {
      throw new Error(`fail no such file or directory ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);

    if (isExists) {
      if (useSync) {
        previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
      } else {
        previewSDK.service.invokeCallback('ok', event, callbackId, {});
      }
    } else {
      throw new Error(`no such file or directory ${filePath}`)
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const accessSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  access(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const appendFile = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { filePath = '', data, encoding } = params;

    if (!/^finfile:\/\/(usr)\//.test(filePath)) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);

    if (isExists) {
      fs__default["default"].appendFileSync(absPath, data, { encoding });
      if (useSync) {
        previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
      } else {
        previewSDK.service.invokeCallback('ok', event, callbackId, {});
      }
    } else {
      throw new Error(`no such file, ${event} ${filePath}`)
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const appendFileSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  appendFile(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const copyFile = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { srcPath = '', destPath = '' } = params;

    if (!/^finfile:\/\/(temp|usr|store)\//.test(srcPath)) {
      throw new Error(`permission denied, open ${srcPath}`)
    }

    if (!destPath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, open ${destPath}`)
    }

    const oldAbsPath = resolveFinfilePath(srcPath, previewSDK.fileCachePath);
    const isOldExists = fs__default["default"].existsSync(oldAbsPath);
    if (!isOldExists) {
      throw new Error(`no such file or directory, ${event} ${srcPath}`)
    }

    const newAbsPath = resolveFinfilePath(destPath, previewSDK.fileCachePath);
    const dir = path__default["default"].dirname(newAbsPath);

    if (!fs__default["default"].existsSync(dir)) {
      throw new Error(`no such file or directory, ${event} ${destPath}`)
    }

    fs__default["default"].copyFileSync(oldAbsPath, newAbsPath);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const copyFileSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  copyFile(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const close$1 = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { fd } = params;

    if (!fd) {
      throw new Error('bad file descriptor')
    }

    fs__default["default"].closeSync(fd);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: 'bad file descriptor' };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const closeSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  close$1(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const fstat = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { fd } = params;

    if (!fd) {
      throw new Error('bad file descriptor')
    }

    const stats = fs__default["default"].fstatSync(fd);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data: stats });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { stats });
    }
  } catch (error) {
    const payload = { errMsg: 'bad file descriptor' };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const fstatSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  fstat(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const ftruncate = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    let { fd, length } = params;

    if (!fd) {
      throw new Error('bad file descriptor')
    }

    if (isNaN(length)) {
      length = 0;
    } else {
      length = Number(length);
    }

    fs__default["default"].ftruncateSync(fd, length);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: 'bad file descriptor' };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const ftruncateSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  ftruncate(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const mkdir = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { dirPath = '', recursive } = params;

    if (!dirPath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, open ${dirPath}`)
    }

    const absPath = resolveFinfilePath(dirPath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);
    if (isExists) {
      throw new Error(`file already exists, ${event} ${dirPath}`)
    }

    fs__default["default"].mkdirSync(absPath, { recursive: Boolean(recursive) });

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const mkdirSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  mkdir(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const open = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { filePath = '', flag = 'r' } = params;

    if (!filePath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);

    const fd = fs__default["default"].openSync(absPath, flag);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data: fd });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { fd });
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const openSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  open(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const read = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    let { fd, arrayBufferLength, offset, length, position } = params;

    if (!fd) {
      throw new Error('bad file descriptor')
    }

    offset = parseNumber(offset, 0);
    length = parseNumber(length, 0);
    position = parseNumber(position, null);

    const buffer$1 = buffer.Buffer.from(new ArrayBuffer(arrayBufferLength));
    const bytesRead = fs__default["default"].readSync(fd, buffer$1, offset, length, position);
    const base64 = buffer.Buffer.from(buffer$1).toString('base64');

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data: { base64, bytesRead } });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { base64, bytesRead });
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const readSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  read(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const readdir = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { dirPath = '' } = params;

    if (!dirPath.startsWith('finfile://usr')) {
      throw new Error(`permission denied, open ${dirPath}`)
    }

    const absPath = resolveFinfilePath(dirPath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);
    if (!isExists) {
      throw new Error(`no such file or directory, ${event} ${dirPath}`)
    }

    const isDir = fs__default["default"].lstatSync(absPath).isDirectory();
    if (!isDir) {
      throw new Error(`not a directory, ${event} ${dirPath}`)
    }

    const list = (fs__default["default"].readdirSync(absPath) || []).map(filePath => {
      filePath = restoreFinfilePath(path__default["default"].resolve(absPath, filePath), previewSDK.fileCachePath);
      filePath = filePath.replace(dirPath, '').replace(/^\//, '');
      return filePath
    });

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data: list });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { files: list });
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const readdirSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  readdir(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const readFile = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    let { filePath, encoding, length, position = 0 } = params;

    // temp 里 file 可以读，但不能写
    if (!/^finfile:\/\/(temp|usr|store)\//.test(filePath)) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);

    if (!fs__default["default"].existsSync(absPath)) {
      throw new Error(`no such file or directory, ${event} ${filePath}`)
    }

    const buffer = fs__default["default"].readFileSync(absPath);
    if (length === undefined) {
      length = buffer.length;
    }

    let data;
    if (encoding === 'utf-8' || encoding === 'utf8') {
      data = buffer.toString(encoding);
      data = data.slice(position, position + length);
    } else {
      data = buffer.toString(encoding, position, position + length);
    }
    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { data });
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const readFileSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  readFile(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const readZipEntry = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  let tempDir;
  try {
    const { filePath, encoding, entries } = params;
    let dirType = 'temp';
    if (filePath.startsWith('finfile://usr/')) {
      dirType = 'usr';
    } else if (filePath.startsWith('finfile://store/')) {
      dirType = 'store';
    }
    // temp 里 file 可以读，但不能写
    // if (!/^finfile:\/\/(usr)/.test(filePath)) {
    //   throw new Error(`permission denied, open ${filePath}`)
    // }
    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isZipExists = fs__default["default"].existsSync(absPath);
    if (!isZipExists || !absPath.endsWith('.zip')) {
      throw new Error(`no such file or directory, ${event} ${filePath}`)
    }

    if (!fs__default["default"].existsSync(absPath)) {
      throw new Error(`no such file or directory, ${event} ${filePath}`)
    }

    const dir = path__default["default"].dirname(absPath);
    tempDir = `${dir}/${uuid.v4()}`;
    await unzipFile(absPath, tempDir);

    function getFileData(filePath, encoding, config) {
      try {
        const size = fs__default["default"].statSync(filePath).size;
        const position = config.position || 0;
        const len = config.length || 0;
        if (len > 52428800) {
          return {
            errMsg: 'readZipEntry:fail the data read is greater than 50M'
          }
        }
        if (position + len > size) {
          return {
            errMsg: 'readZipEntry:fail the value of "length" is out of range'
          }
        }
        return {
          errMsg: 'readZipEntry:ok',
          url: `finrequest://readzipentry?path=${encodeURIComponent(
            path__default["default"].join(path__default["default"].basename(tempDir), path__default["default"].relative(tempDir, filePath))
          )}&start=${position}&len=${len}&size=${size}&encoding=${encoding}&dirType=${dirType}`,
          encoding: encoding
        }
      } catch (error) {
        return {
          errMsg: 'readZipEntry:fail ' + error.toString()
        }
      }
    }

    const res = {};
    if (entries instanceof Array) {
      entries.forEach(({ path: filePath, encoding = 'base64', position: start, length: len }) => {
        res[filePath] = getFileData(path__default["default"].resolve(tempDir, filePath), encoding, { position: start, length: len });
      });
    } else {
      const paths = fs__default["default"].readdirSync(tempDir).map(currentPath => path__default["default"].resolve(tempDir, currentPath));
      while (paths.length > 0) {
        const currentAbsPath = paths.shift();
        const isDir = fs__default["default"].lstatSync(currentAbsPath).isDirectory();
        if (isDir) {
          paths.push(...fs__default["default"].readdirSync(currentAbsPath).map(currentPath => path__default["default"].resolve(currentAbsPath, currentPath)));
        } else {
          const filePath = path__default["default"].relative(tempDir, currentAbsPath).replace(/\\/g, '/');

          if (fs__default["default"].statSync(currentAbsPath).size > 52428800) {
            res[filePath] = {
              errMsg: 'readZipEntry:fail the data read is greater than 50M'
            };
          } else {
            res[filePath] = getFileData(currentAbsPath, encoding, { position: 0, length: 0 });
          }
        }
      }
    }

    previewSDK.service.invokeCallback('ok', event, callbackId, { entries: res });
  } catch (error) {
    const payload = { errMsg: error.toString() };
    previewSDK.service.invokeCallback('fail', event, callbackId, payload);
  }
  // tempDir && fs.rmdirSync(tempDir, { recursive: true })
};

const rename = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { oldPath = '', fatNewPath = '' } = params;

    if (!/^finfile:\/\/(temp|usr|store)\//.test(oldPath)) {
      throw new Error(`permission denied, open ${oldPath}`)
    }

    if (!fatNewPath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, open ${fatNewPath}`)
    }

    const oldAbsPath = resolveFinfilePath(oldPath, previewSDK.fileCachePath);
    const isOldExists = fs__default["default"].existsSync(oldAbsPath);
    if (!isOldExists) {
      throw new Error(`no such file or directory, ${event} ${oldPath}`)
    }

    const newAbsPath = resolveFinfilePath(fatNewPath, previewSDK.fileCachePath);
    const dir = path__default["default"].dirname(newAbsPath);

    if (!fs__default["default"].existsSync(dir)) {
      fs__default["default"].mkdirSync(dir, { recursive: true });
    }

    fs__default["default"].renameSync(oldAbsPath, newAbsPath);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const renameSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  rename(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const rmdir = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { dirPath = '', recursive } = params;

    if (dirPath === 'finfile://usr/' || !dirPath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, open ${dirPath}`)
    }

    const absPath = resolveFinfilePath(dirPath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);
    if (!isExists) {
      throw new Error(`no such directory, ${event} ${dirPath}`)
    }

    const isDir = fs__default["default"].lstatSync(absPath).isDirectory();
    if (!isDir) {
      throw new Error(`no such directory, ${event} ${dirPath}`)
    }

    fs__default["default"].rmdirSync(absPath, { recursive: Boolean(recursive) });

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const rmdirSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  rmdir(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const saveFile = (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) => {
  try {
    const { tempFilePath, filePath } = params;

    if (!tempFilePath.startsWith('finfile://temp/')) {
      throw new Error(`permission denied, open ${tempFilePath}`)
    }

    if (filePath && !filePath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(tempFilePath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);
    if (!isExists) {
      throw new Error(`tempFilePath file not exist, ${event} ${tempFilePath}`)
    }

    let newAbsPath;
    if (!filePath) {
      const destFileName = generateFileName(absPath);
      newAbsPath = path__default["default"].join(previewSDK.fileCachePath, 'store', destFileName);
    } else {
      newAbsPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
      const dir = path__default["default"].dirname(newAbsPath);
      if (!fs__default["default"].existsSync(dir)) {
        throw new Error(`no such file or directory, ${dir}`)
      }
    }

    fs__default["default"].copyFileSync(absPath, newAbsPath);
    fs__default["default"].removeSync(absPath);
    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data: restoreFinfilePath(newAbsPath, previewSDK.fileCachePath) });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { savedFilePath: restoreFinfilePath(newAbsPath, previewSDK.fileCachePath) });
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const saveFileSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  saveFile(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const stat = (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) => {
  try {
    const { path: filePath, recursive } = params;

    const useRecursive = !!recursive;

    if (!filePath.startsWith('finfile://usr')) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);
    if (!isExists) {
      throw new Error(`no such file or directory, ${event} ${filePath}`)
    }

    const isDir = fs__default["default"].lstatSync(absPath).isDirectory();

    /**
     * @type {statObject|{path:string, stats?:statObject}[]}
     */
    let res;

    if (useRecursive && isDir) {
      res = [];

      const paths = fs__default["default"].readdirSync(absPath).map(currentPath => path__default["default"].resolve(absPath, currentPath));
      while (paths.length > 0) {
        const currentAbsPath = paths.shift();
        const isSubDir = fs__default["default"].lstatSync(currentAbsPath).isDirectory();
        if (isSubDir) {
          paths.push(...fs__default["default"].readdirSync(currentAbsPath).map(currentPath => path__default["default"].resolve(currentAbsPath, currentPath)));
        }
        const fsStat = fs__default["default"].statSync(absPath, { throwIfNoEntry: false });
        const stats = fsStat && fsStatToFinclipStat(fsStat);
        res.push({ path: currentAbsPath.replace(previewSDK.fileCachePath + '/usr', ''), stats });
      }
    } else {
      res = fsStatToFinclipStat(fs__default["default"].statSync(absPath));
    }

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data: res });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { stats: res });
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const statSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  stat(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const truncate = (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) => {
  try {
    let { filePath, length } = params;

    if (!filePath.startsWith('finfile://usr')) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    if (!fs__default["default"].existsSync(absPath) || fs__default["default"].lstatSync(absPath).isDirectory()) {
      throw new Error(`no such file or directory, ${event} ${filePath}`)
    }

    if (isNaN(length)) {
      length = 0;
    } else {
      length = Number(length);
    }

    fs__default["default"].truncateSync(absPath, length);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const truncateSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  truncate(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const unlink = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { filePath = '' } = params;

    if (!/^finfile:\/\/(usr|store)\//.test(filePath)) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const isExists = fs__default["default"].existsSync(absPath);
    if (!isExists) {
      throw new Error(`no such file or directory, ${event} ${filePath}`)
    }

    const isDir = fs__default["default"].lstatSync(absPath).isDirectory();
    if (isDir) {
      throw new Error(`operation not permitted, ${event} ${filePath}`)
    }

    fs__default["default"].unlinkSync(absPath);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const unlinkSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  unlink(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const unzip = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { zipFilePath = '', targetPath = '' } = params;

    if (!/^finfile:\/\/(usr|store)\//.test(zipFilePath) || !targetPath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, unzip ${zipFilePath} -> ${targetPath}`)
    }

    const zipAbsPath = resolveFinfilePath(zipFilePath, previewSDK.fileCachePath);
    const isZipExists = fs__default["default"].existsSync(zipAbsPath);
    if (!isZipExists || !zipAbsPath.endsWith('.zip')) {
      throw new Error(`no such file or directory, ${event} ${zipFilePath}`)
    }

    const targetAbsPath = resolveFinfilePath(targetPath, previewSDK.fileCachePath);
    const dir = path__default["default"].dirname(targetAbsPath);

    if (!fs__default["default"].existsSync(dir)) {
      throw new Error(`no such file or directory, unzip ${zipFilePath} -> ${targetPath}`)
    }

    await unzipFile(zipAbsPath, targetAbsPath);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const write = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    let { fd, data, offset, length, position, encoding = 'utf-8' } = params;

    if (!fd) {
      throw new Error('bad file descriptor')
    }

    offset = parseNumber(offset, 0);
    position = parseNumber(position, null);

    const buffer$1 = buffer.Buffer.from(data, encoding);

    length = parseNumber(length, buffer$1.byteLength - offset);

    const bytesWritten = fs__default["default"].writeSync(fd, buffer$1, offset, length, position);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data: bytesWritten });
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, { bytesWritten });
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const writeSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  write(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const writeFile = async function (event, callbackId, params, ipcEvent, previewSDK, { useSync = false } = {}) {
  try {
    const { filePath, data, encoding } = params;

    if (filePath === 'finfile://usr') {
      throw new Error(`illegal operation on a directory, open ${filePath}`)
    }

    if (!filePath.startsWith('finfile://usr/')) {
      throw new Error(`permission denied, open ${filePath}`)
    }

    const absPath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    const dir = path__default["default"].dirname(absPath);

    if (!fs__default["default"].existsSync(dir)) {
      fs__default["default"].mkdirSync(dir, { recursive: true });
    }
    const buffer$1 = buffer.Buffer.from(data, encoding);
    fs__default["default"].writeFileSync(absPath, buffer$1);

    if (useSync) {
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
    } else {
      previewSDK.service.invokeCallback('ok', event, callbackId, {});
    }
  } catch (error) {
    const payload = { errMsg: error.toString().replace(previewSDK.fileCachePath + '/', 'finfile://') };
    if (useSync) {
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, payload);
    } else {
      previewSDK.service.invokeCallback('fail', event, callbackId, payload);
    }
  }
};

const writeFileSync = function (event, callbackId, params, ipcEvent, previewSDK) {
  writeFile(event, callbackId, params, ipcEvent, previewSDK, { useSync: true });
};

const fileHandler = {
  removeSavedFile,
  openDocument: openDocument$1,
  getFileInfo,
  getSavedFileList,
  getSavedFileInfo,
  fileToBase64,
  base64ToTempFilePath,

  // file system 的 api
  access,
  accessSync,
  appendFile,
  appendFileSync,
  copyFile,
  copyFileSync,
  close: close$1,
  closeSync,
  fstat,
  fstatSync,
  ftruncate,
  ftruncateSync,
  mkdir,
  mkdirSync,
  open,
  openSync,
  read,
  readSync,
  readdir,
  readdirSync,
  readFile,
  readFileSync,
  readZipEntry,
  rename,
  renameSync,
  rmdir,
  rmdirSync,
  saveFile,
  saveFileSync,
  stat,
  statSync,
  truncate,
  truncateSync,
  unlink,
  unlinkSync,
  unzip,
  write,
  writeSync,
  writeFile,
  writeFileSync,
};

const getSelectedTextRange = (event, callbackId, params, ipcEvent, previewSDK) => {
  const uuid$1 = uuid.v4();
  if (!previewSDK.route.currentView) {
    return previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: '找不到当前的 view' })
  }
  previewSDK.route.currentView.event.once(`getSelectedTextRange-${uuid$1}`, (res) => {
    if (!res) {
      return previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: '找不到 focus 的 input' })
    }
    const { start, end } = res;
    previewSDK.service.invokeCallback('ok', event, callbackId, {
      start,
      end
    });
  });
  previewSDK.route.currentView.send('getSelectedTextRange', {
    uuid: uuid$1
  });
};

const inputHandler = {
  getSelectedTextRange
};

const canvasMethod = (event, callbackId, params, ipcEvent, previewSDK) => {
  const uuid$1 = uuid.v4();
  if (!previewSDK.route.currentView) {
    return previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: '找不到当前的 view' })
  }
  previewSDK.route.currentView.event.once(`canvasMethod-${uuid$1}`, (res) => {
    if (!res) {
      return previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: `${params.method} 执行失败` })
    }
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, res);
  });
  previewSDK.route.currentView.send('canvasMethod', {
    uuid: uuid$1,
    method: params.method,
    data: params.data
  });
};

const canvasHandler = {
  canvasMethod
};

let latitude$1 = 22.93772;
let longitude$1 = 22.93772;

electron.ipcMain.on('setLocationInfo', (e, payload) => {
  latitude$1 = parseFloat(payload.latitude) === 0 ? 0 : (parseFloat(payload.latitude) || 22.93772);
  longitude$1 = parseFloat(payload.longitude) === 0 ? 0 : (parseFloat(payload.longitude) || 22.93772);
});

const getLocation$1 = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {
    accuracy: 65,
    horizontalAccuracy: 65,
    latitude: latitude$1,
    longitude: longitude$1,
    speed: -1,
    verticalAccuracy: 65,
  });
};

const openLocation = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    previewSDK._emitSdkEvent('openLocation', { ...Object.assign({ address: '' }, params), });
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const chooseLocation$1 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const locationId = uuid.v4();
    previewSDK._emitSdkEvent('chooseLocation', { ...Object.assign({ latitude: {}, longitude: {} }, params), locationId });
    const response = await new Promise((resolve, reject) => {
      previewSDK.once(`chooseLocation-${locationId}`, (res) => {
        if (res.cancel) {
          reject('cancel');
        } else {
          resolve(res);
        }
      });
    });
    previewSDK.service.invokeCallback('ok', event, callbackId, response.data);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const choosePoi = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const choosepoiId = uuid.v4();
    previewSDK._emitSdkEvent('choosePoi', { choosepoiId });
    const response = await new Promise((resolve, reject) => {
      previewSDK.once(`choosePoi-${choosepoiId}`, (res) => {
        if (res.cancel) {
          reject('cancel');
        } else {
          resolve(res);
        }
      });
    });
    previewSDK.service.invokeCallback('ok', event, callbackId, response.data);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const localtionHandler = {
  getLocation: getLocation$1,
  openLocation,
  chooseLocation: chooseLocation$1,
  choosePoi
};

const getNetworkType = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    previewSDK.service.invokeCallback('ok', event, callbackId, {
      networkType: 'wifi',
    });
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const networkHandle$1 = {
  getNetworkType
};

const showToast = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('showToast', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const hideToast = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('hideToast', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const showLoading = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('showLoading', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const hideLoading = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('hideLoading', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const showModal = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    const modalId = Date.now();
    previewSDK._emitSdkEvent('showModal', { ...params, modalId });
    const response = await new Promise((resolve, reject) => {
      previewSDK.homeEventManager.once(`showModalRes-${modalId}`, (res) => {
        resolve(res);
      });
    });
    previewSDK.service.invokeCallback('ok', event, callbackId, response);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const showActionSheet = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    const actionSheetId = Date.now();
    previewSDK._emitSdkEvent('showActionSheet', { ...params, actionSheetId });
    const response = await new Promise((resolve, reject) => {
      previewSDK.homeEventManager.once(`showActionSheetRes-${actionSheetId}`, (res) => {
        resolve(res);
      });
    });
    if (response.cancel) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'cancel' });
      return
    }
    if (typeof response.tapIndex === 'number') {
      previewSDK.service.invokeCallback('ok', event, callbackId, {
        tapIndex: response.tapIndex
      });
    }
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const isCustomTabbar = (ftConfig) => {
  return ftConfig && ftConfig.tabBar && ftConfig.tabBar.custom
};

const setTabBarItem = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (typeof params.index !== 'number') throw new Error('index must be number')
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('setTabBarItem', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setTabBarStyle = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('setTabBarStyle', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const showTabBarRedDot = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (typeof params.index !== 'number') throw new Error('index must be number')
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('showTabBarRedDot', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};
const hideTabBarRedDot = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (typeof params.index !== 'number') throw new Error('index must be number')
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('hideTabBarRedDot', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setTabBarBadge = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (typeof params.index !== 'number') throw new Error('index must be number')
    if (typeof params.text !== 'string') throw new Error('text must be string')
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('setTabBarBadge', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};
const removeTabBarBadge = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (typeof params.index !== 'number') throw new Error('index must be number')
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('removeTabBarBadge', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const showTabBar = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('showTabBar', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};
const hideTabBar = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    if (isCustomTabbar(previewSDK.ftConfig)) throw new Error('custom Tabbar')
    previewSDK._emitSdkEvent('hideTabBar', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setBackgroundTextStyle = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    previewSDK._emitSdkEvent('setBackgroundTextStyle', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setBackgroundColor = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    previewSDK._emitSdkEvent('setBackgroundColor', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const startPullDownRefresh = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    if (params.webviewIds && params.webviewIds.length) {
      params.webviewIds.forEach(webviewId => {
        if (previewSDK.route && previewSDK.route.stack.length) {
          const view = previewSDK.route.stack.find(viewItem => viewItem.webviewId === webviewId);
          view && view.send('startPullDownRefresh');
        }
      });
    }
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const stopPullDownRefresh = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    if (params.webviewIds && params.webviewIds.length) {
      params.webviewIds.forEach(webviewId => {
        if (previewSDK.route && previewSDK.route.stack.length) {
          const view = previewSDK.route.stack.find(viewItem => viewItem.webviewId === webviewId);
          view && view.send('stopPullDownRefresh');
        }
      });
    }
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const getMenuButtonBoundingClientRect = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    const requestId = Date.now();
    previewSDK._emitSdkEvent('getMenuButtonBoundingClientRect', { ...params, requestId });
    const response = await new Promise((resolve, reject) => {
      previewSDK.once(`getMenuButtonBoundingClientRect-${requestId}`, (res) => {
        resolve(res);
      });
    });
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, response);
  } catch (error) {
    previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: error.toString() });
  }
};

const alertBeforeUnload = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('alertBeforeUnload', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

var interfaceApis = {
  showToast,
  hideToast,
  showLoading,
  hideLoading,
  showModal,
  showActionSheet,
  setTabBarItem,
  setTabBarStyle,
  showTabBarRedDot,
  hideTabBarRedDot,
  setTabBarBadge,
  removeTabBarBadge,
  showTabBar,
  hideTabBar,
  setBackgroundColor,
  setBackgroundTextStyle,
  startPullDownRefresh,
  stopPullDownRefresh,
  getMenuButtonBoundingClientRect,
  alertBeforeUnload
};

const Jimp = require('jimp');

const saveImageToPhotosAlbum = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  let filePath = params.filePath;
  if (filePath.startsWith('.') || filePath.startsWith('/')) {
    filePath = filePath.startsWith('/') ? `${getProjectRoot()}${filePath}` : `${getProjectRoot()}/${filePath}`;
  } else if (isFinfile(filePath)) {
    filePath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
  }
  electron.dialog
    .showSaveDialog(previewSDK.homeWin, {
      title: '保存文件',
      defaultPath: filePath,
    })
    .then((result) => {
      if (result.canceled) {
        previewSDK.service.invokeCallback('fail', event, callbackId, {
          errMsg: 'cancel',
        });
      }
      if (result.filePath) {
        fs__default["default"].copyFileSync(filePath, result.filePath);
        previewSDK.service.invokeCallback('ok', event, callbackId, {});
      }
    });
};

const previewImage = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('previewMedia', {
    type: 'image',
    ...params
  });
  previewSDK.service.invokeCallback('ok', event, callbackId, {}, ipcEvent, previewSDK);
};

const getImageInfo = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  const src = params.src;
  let filePath;
  if (isFinfile(src)) {
    filePath = resolveFinfilePath(src, previewSDK.fileCachePath);
  } else {
    filePath = path__default["default"].join(previewSDK.path, src);
  }

  if (fs__default["default"].existsSync(filePath)) {
    try {
      const image = await Jimp.read(filePath);
      previewSDK.service.invokeCallback('ok', event, callbackId, {
        path: src,
        width: image.bitmap.width,
        height: image.bitmap.height,
        orientation: 'up',
        type: image._originalMime.split('/')[1]
      });
    } catch (error) {
      previewSDK.service.invokeCallback('fail', event, callbackId, {
        errMsg: `getImageInfo:fail ${error}`
      });
    }
  } else {
    previewSDK.service.invokeCallback('fail', event, callbackId, {
      errMsg: 'getImageInfo:fail file not found'
    });
  }
};

const compressImage = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  const { src, quality } = params;
  let filePath;
  if (isFinfile(src)) {
    filePath = resolveFinfilePath(src, previewSDK.fileCachePath);
  } else {
    filePath = path__default["default"].join(previewSDK.path, src);
  }

  if (fs__default["default"].existsSync(filePath)) {
    const filename = generateFileName(filePath);
    const tempFilePath = `finfile://temp/${filename}`;
    const targetPath = resolveFinfilePath(tempFilePath, previewSDK.fileCachePath);
    const image = await Jimp.read(filePath);
    await image.quality(quality);

    // mark: gif 压缩先处理成 jps, 再 rename 为 gif, 一是没有好用的 gif 压缩库、二也是对齐微信压缩 gif 的表现
    const isGIF = /\.gif/.test(targetPath);
    if (isGIF) {
      const temp = targetPath.replace('.gif', '.jpg');
      await image.writeAsync(temp);
      fs__default["default"].renameSync(temp, targetPath);
    } else {
      await image.write(targetPath);
    }
    previewSDK.service.invokeCallback('ok', event, callbackId, {
      width: image.bitmap.width,
      height: image.bitmap.height,
      filePath,
      quality,
      tempFilePath
    });
  } else {
    previewSDK.service.invokeCallback('fail', event, callbackId, {
      quality,
      filePath,
      errMsg: "compressImage:fail file doesn't exist"
    });
  }
};

const chooseImage$1 = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  if (isNaN(params.count)) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: `非法参数 count ${params.count}` });
    return
  }
  try {
    const response = await electron.dialog.showOpenDialog(previewSDK.homeWin, {
      title: '打开',
      properties: ['openFile', 'multiSelections'],
      filters: [{ name: 'Images', extensions: ['jpg', 'jpeg', 'png', 'gif', 'svg'] }]
    });

    if (response.canceled) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'cancel' });
      return
    }
    const result = {
      tempFilePaths: [],
      tempFiles: [],
    };
    if (response.filePaths && response.filePaths.length) {
      response.filePaths.slice(0, params.count).forEach((file) => {
        const tempFile = saveFileToTempPath(file, previewSDK.fileCachePath);
        if (typeof tempFile === 'object') {
          result.tempFilePaths.push(tempFile.path);
          result.tempFiles.push(tempFile);
        }
      });
    }
    previewSDK.service.invokeCallback('ok', event, callbackId, result);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const chooseMessageFile = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  if (isNaN(params.count)) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: `非法参数 count ${params.count}` });
    return
  }

  const images = ['jpg', 'jpeg', 'png', 'gif', 'svg', 'webp'];
  const videos = ['mkv', 'avi', 'mp4', 'flv', 'rmvb', 'mov'];

  try {
    let filters = [];
    switch (params.type) {
      case 'image':
        filters = [{ name: 'Images', extensions: images }];
        break
      case 'video':
        filters = [{ name: 'Videos', extensions: videos }];
        break
      case 'file':
        filters = [{ name: 'Files', extensions: params.extensions || ['*'] }];
        break
      default:
        filters = [{ name: 'All Files', extensions: ['*'] }];
        break
    }

    const response = await electron.dialog.showOpenDialog(previewSDK.homeWin, {
      title: '打开',
      properties: ['openFile', 'multiSelections'],
      filters
    });

    if (response.canceled) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'cancel' });
      return
    }
    const result = {
      tempFiles: [],
    };
    if (response.filePaths && response.filePaths.length) {
      response.filePaths.slice(0, params.count).forEach((file) => {
        const info = path__default["default"].basename(file);
        const ext = path__default["default"].extname(info);
        const extname = ext.slice(1);
        const name = info.replace(ext, '');
        let type = 'file';
        if (images.includes(extname)) {
          type = 'image';
        }
        if (videos.includes(extname)) {
          type = 'video';
        }

        const tempFile = saveFileToTempPath(file, previewSDK.fileCachePath);
        if (typeof tempFile === 'object') {
          result.tempFiles.push({
            ...tempFile,
            name,
            type,
            time: Math.floor(Date.now() / 1000)
          });
        }
      });
    }
    previewSDK.service.invokeCallback('ok', event, callbackId, result);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

var image = {
  saveImageToPhotosAlbum,
  previewImage,
  getImageInfo,
  compressImage,
  chooseImage: chooseImage$1,
  chooseMessageFile
};

/**
   * @param {import('#/index').default} previewSDK 
   */
const playVoice = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { filePath, duration } = params;
    if (duration && typeof duration !== 'number') throw new TypeError('duration must be number')
    if (typeof duration === 'number' &&  duration <= 0) throw new RangeError('duration must be greater than zero')
    let fileAbsolutePath;
    if (isFinfile(filePath)) {
      fileAbsolutePath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    } else {
      fileAbsolutePath = filePath;
    }
    if (!fs__default["default"].existsSync(fileAbsolutePath)) throw new Error('无效文件')

    previewSDK._emitSdkEvent('playVoice', { audioFileDir: fileAbsolutePath, duration });
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const pauseVoice = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('pauseVoice', {});
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const stopVoice = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('stopVoice', {});
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const operateMusicPlayer = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('operateMusicPlayer', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const getMusicPlayerState = async (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('getMusicPlayerState', { ...params, callbackId });
  const response = await new Promise((resolve, reject) => {
    previewSDK.once(`getMusicPlayerStateRes-${callbackId}`, (res) => {
      resolve(res);
    });
  });
  previewSDK.service.invokeCallback('ok', event, callbackId, response);
};

const backgroundAudioManager = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('backgroundAudioManager', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const innerAudio = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('innerAudio', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

var voice = {
  playVoice,
  pauseVoice,
  stopVoice,
  operateMusicPlayer,
  getMusicPlayerState,
  backgroundAudioManager,
  innerAudio
};

const saveVideoToPhotosAlbum = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  let filePath = params.filePath;
  if (isFinfile(filePath)) {
    filePath = resolveFinfilePath(filePath, previewSDK.fileCachePath);
    electron.dialog.showSaveDialog(previewSDK.homeWin, {
      title: '保存文件',
      defaultPath: filePath
    }).then((result) => {
      if (result.canceled) {
        previewSDK.service.invokeCallback('fail', event, callbackId, {
          errMsg: 'cancel'
        });
      }
      if (result.filePath) {
        fs__default["default"].copyFileSync(filePath, result.filePath);
        previewSDK.service.invokeCallback('ok', event, callbackId, {});
      }
    });
  } else {
    previewSDK.service.invokeCallback('fail', event, callbackId, {
      errMsg: `${filePath} not absolute path`,
    });
  }
};

const previewMedia = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('previewMedia', {
    type: 'media',
    ...params
  });
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const chooseVideo = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    const response = await electron.dialog.showOpenDialog(previewSDK.homeWin, {
      title: '打开',
      properties: ['openFile'],
      filters: [{ name: 'Movies', extensions: ['*'] }]
    });

    if (response.canceled) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'cancel' });
      return
    }

    const result = {
      width: 0,
      height: 0,
      size: 0,
      duration: 0,
      tempFilePath: '',
      thumbTempFilePath: ''
    };

    if (response.filePaths && response.filePaths.length) {
      const file = response.filePaths[0];
      const { path: filePath, size } = saveFileToTempPath(file, previewSDK.fileCachePath);

      const videoId = Date.now();
      previewSDK._emitSdkEvent('getMediaInfo', {
        videoId,
        filePath,
      });
      previewSDK.once(`getMediaInfoRes-${videoId}`, (info) => {
        let { src } = info;
        src = src.replace(/^data:image\/\w+;base64,/, '');
        const dataBuffer = Buffer.from(src, 'base64');
        const thumbTempFilePath = `finfile://temp/${generateFileName('thumb.png')}`;
        fs__default["default"].writeFileSync(resolveFinfilePath(thumbTempFilePath, previewSDK.fileCachePath), dataBuffer);
        result.width = info.width;
        result.height = info.height;
        result.duration = info.duration;
        result.size = size;
        result.tempFilePath = filePath;
        result.thumbTempFilePath = thumbTempFilePath;
        previewSDK.service.invokeCallback('ok', event, callbackId, result);
      });
    }
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const chooseMedia = async (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    const type = {
      image: { name: 'Images', extensions: ['jpg', 'jpeg', 'png', 'svg', 'webp', 'gif'] },
      video: { name: 'Movies', extensions: ['mp4', 'mov', 'm4v', '3gp', 'avi', 'mky', 'webm'] }
    };

    // filters 数组设置两个 filter 不会生效，把两个 filter 的 extensions 整合在一块
    const filter = { name: 'File', extensions: [] };
    let arr = ['image', 'video'];
    const isMix = params.mediaType.indexOf('mix') > -1;
    if (params.mediaType) {
      arr = isMix ? ['image', 'video'] : params.mediaType;
    }
    arr.forEach((mediaType) => {
      const { extensions = [] } = type[mediaType] || {};
      filter.extensions = filter.extensions.concat(extensions);
    });
    const response = await electron.dialog.showOpenDialog(previewSDK.homeWin, {
      title: '打开',
      properties: ['openFile', 'multiSelections'],
      filters: [filter]
    });

    if (response.canceled) {
      previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'cancel' });
      return
    }

    const result = {
      width: 0,
      height: 0,
      size: 0,
      duration: 0,
      tempFilePath: '',
      thumbTempFilePath: ''
    };

    const obj = {
      type: '',
      tempFiles: []
    };

    if (response.filePaths && response.filePaths.length) {
      const firstTypeIsImage = type.image.extensions.indexOf(path__default["default"].extname(response.filePaths[0]).replace('.', '').toLowerCase()) > -1;
      Promise.all(response.filePaths.slice(0, params.count).map(file => {
        const { path: filePath, size } = saveFileToTempPath(file, previewSDK.fileCachePath);

        if (type.image.extensions.indexOf(path__default["default"].extname(filePath).replace('.', '').toLowerCase()) > -1) {
          if (!firstTypeIsImage && !isMix) {
            return Promise.resolve()
          }
          obj.tempFiles.push({
            fileType: 'image',
            size,
            tempFilePath: filePath
          });
          obj.type = /mix|video/.test(obj.type) ? 'mix' : 'image';
          return Promise.resolve()
        }

        if (firstTypeIsImage && !isMix) {
          return Promise.resolve()
        }
        const videoId = Date.now();
        previewSDK._emitSdkEvent('getMediaInfo', {
          videoId,
          filePath,
        });
        return new Promise((resolve, reject) => {
          previewSDK.once(`getMediaInfoRes-${videoId}`, (info) => {
            let { src } = info;
            if (!src) {
              reject(new Error('invalid file'));
              return
            }
            src = src.replace(/^data:image\/\w+;base64,/, '');
            const dataBuffer = Buffer.from(src, 'base64');
            const thumbTempFilePath = `finfile://temp/${generateFileName('temp.png')}`;
            fs__default["default"].writeFileSync(resolveFinfilePath(thumbTempFilePath, previewSDK.fileCachePath), dataBuffer);
            const fileInfo = { ...result };
            fileInfo.fileType = 'video';
            fileInfo.width = info.width;
            fileInfo.height = info.height;
            fileInfo.duration = info.duration;
            fileInfo.size = size;
            fileInfo.tempFilePath = filePath;
            fileInfo.thumbTempFilePath = thumbTempFilePath;
            obj.tempFiles.push(fileInfo);
            obj.type = /mix|image/.test(obj.type) ? 'mix' : 'video';
            resolve();
          });
        })
      })).then(() => {
        previewSDK.service.invokeCallback('ok', event, callbackId, obj);
      }).catch((error) => {
        previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
      });
    }
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

var video = {
  saveVideoToPhotosAlbum,
  previewMedia,
  chooseVideo,
  chooseMedia
};

var media = {
  ...image,
  ...voice,
  ...video
};

const showNavigationBarLoading = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    previewSDK._emitSdkEvent('showNavigationBarLoading', true);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setNavigationBarTitle$1 = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    previewSDK._emitSdkEvent('setNavigationBarTitle', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const hideNavigationBarLoading = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    previewSDK._emitSdkEvent('hideNavigationBarLoading', false);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setNavigationBarColor = (event, callbackId, params = {}, ipcEvent, previewSDK) => {
  try {
    previewSDK._emitSdkEvent('setNavigationBarColor', params);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

var navigationBar = {
  setNavigationBarTitle: setNavigationBarTitle$1,
  showNavigationBarLoading,
  hideNavigationBarLoading,
  setNavigationBarColor
};

const MAX_VALUE_LENGTH = 1024 * 1024;
const MAX_DATA_LENGTH = 10 * 1024 * 1024;

const getStoragePath = (previewSDK) => path__default["default"].join(previewSDK.storagePath, 'storage.json');

const setStorage$2 = (key, data, previewSDK) => {
  const storagePath = getStoragePath(previewSDK);
  if (typeof key !== 'string') throw new Error('key 必须是string')
  if (typeof data === 'undefined') throw new Error('data不能为空')
  if (JSON.stringify(data).length > MAX_VALUE_LENGTH) throw new Error('value 长度不能大于1Mb')
  fs__default["default"].ensureFileSync(storagePath);
  let dataObj = {};
  try {
    dataObj = JSON.parse(fs__default["default"].readFileSync(storagePath, 'utf-8') || '{}');
  } catch (error) {
    fs__default["default"].removeSync(storagePath);
    throw error
  }
  dataObj[key] = data;
  const dataStr = JSON.stringify(dataObj);
  if (dataStr.length > MAX_DATA_LENGTH) throw new Error('单个小程序storage总大小大不能大于10Mb')
  fs__default["default"].writeFileSync(storagePath, dataStr);
  return true
};

const removeStorage$2 = (key, previewSDK) => {
  const storagePath = getStoragePath(previewSDK);
  fs__default["default"].ensureFileSync(storagePath);
  const dataObj = JSON.parse(fs__default["default"].readFileSync(storagePath, 'utf-8') || '{}');
  delete dataObj[key];
  const dataStr = JSON.stringify(dataObj);
  fs__default["default"].writeFileSync(storagePath, dataStr);
  return true
};

const getStorage$2 = (key, previewSDK) => {
  const storagePath = getStoragePath(previewSDK);
  fs__default["default"].ensureFileSync(storagePath);
  const dataObj = JSON.parse(fs__default["default"].readFileSync(storagePath, 'utf-8') || '{}');
  return dataObj[key]
};

const getStorageInfo$2 = (previewSDK) => {
  const storagePath = getStoragePath(previewSDK);
  fs__default["default"].ensureFileSync(storagePath);
  const dataStr = fs__default["default"].readFileSync(storagePath, 'utf-8');
  const currentSize = parseInt(dataStr.length / 1024) + 1;
  const dataObj = JSON.parse(dataStr || '{}');
  return {
    currentSize,
    keys: Object.keys(dataObj),
    limitSize: 10240,
  }
};

const clearStorage$2 = (previewSDK) => {
  const storagePath = getStoragePath(previewSDK);
  if (fs__default["default"].existsSync(storagePath)) {
    fs__default["default"].removeSync(storagePath);
  }
  return true
};

var storageUtils = {
  getStoragePath,
  setStorage: setStorage$2,
  removeStorage: removeStorage$2,
  getStorage: getStorage$2,
  getStorageInfo: getStorageInfo$2,
  clearStorage: clearStorage$2
};

const setStorage$1 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { key, data } = params;
    storageUtils.setStorage(key, data, previewSDK);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setStorageSync = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { key, data } = params;
    storageUtils.setStorage(key, data, previewSDK);
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
  } catch (error) {
    previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: error.toString() });
  }
};

const removeStorage$1 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { key } = params;
    storageUtils.removeStorage(key, previewSDK);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const removeStorageSync = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { key } = params;
    storageUtils.removeStorage(key, previewSDK);
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, {});
  } catch (error) {
    previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: error.toString() });
  }
};

const getStorage$1 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { key } = params;
    const data = storageUtils.getStorage(key, previewSDK);
    if (!data) previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: 'data not found' });
    previewSDK.service.invokeCallback('ok', event, callbackId, { data });
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const getStorageSync = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { key } = params;
    const data = storageUtils.getStorage(key, previewSDK) || '';
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { data });
  } catch (error) {
    previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: error.toString() });
  }
};

const getStorageInfo$1 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const info = storageUtils.getStorageInfo(previewSDK);
    previewSDK.service.invokeCallback('ok', event, callbackId, info, previewSDK);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const getStorageInfoSync = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const info = storageUtils.getStorageInfo(previewSDK);
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, info);
  } catch (error) {
    previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: error.toString() });
  }
};

const clearStorage$1 = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    storageUtils.clearStorage(previewSDK);
    previewSDK.service.invokeCallback('ok', event, callbackId, {});
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const clearStorageSync = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    storageUtils.clearStorage(previewSDK);
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { });
  } catch (error) {
    previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: error.toString() });
  }
};

var storage = {
  setStorage: setStorage$1,
  setStorageSync,
  removeStorage: removeStorage$1,
  removeStorageSync,
  getStorage: getStorage$1,
  getStorageSync,
  getStorageInfo: getStorageInfo$1,
  getStorageInfoSync,
  clearStorage: clearStorage$1,
  clearStorageSync
};

const showShareMenu = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('showShareMenu', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const hideShareMenu = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK._emitSdkEvent('hideShareMenu', params);
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const menuButtonList = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

var shareMenu = {
  showShareMenu,
  hideShareMenu,
  menuButtonList,
};

const navigateToMiniProgram$1 = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const navigateBackMiniProgram$1 = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const exitMiniProgram = (event, callbackId, params, ipcEvent, previewSDK) => {
  previewSDK.service.invokeCallback('ok', event, callbackId, {});
};

const otherAPIHandler = {
  navigateToMiniProgram: navigateToMiniProgram$1,
  navigateBackMiniProgram: navigateBackMiniProgram$1,
  exitMiniProgram
};

electron.ipcMain.handle('askForMediaAccess', async (event, mediaType) => {
  if (process.platform === 'darwin') {
    const res = await electron.systemPreferences.askForMediaAccess(mediaType);
    return res
  } else {
    return true
  }
});

const cameraContextInvoke = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { cname, webviewId } = params;
    const invokeId = uuid.v4();
    params.invokeId = invokeId;

    const view = previewSDK.route.getView(webviewId);
    view.send('cameraContextInvoke', params);

    electron.ipcMain.once(`cameraContextInvoke-${invokeId}`, (e, res) => {
      if (res.error) throw new Error(res.error)

      if (cname === 'takePhoto') {
        previewSDK.service.invokeCallback('ok', event, callbackId, {
          tempImagePath: `finfile://temp/${res.fileName}`,
        });
        return
      }

      if (cname === 'setZoom') {
        previewSDK.service.invokeCallback('ok', event, callbackId, {
          zoom: res.zoom,
        });
        return
      }

      if (cname === 'startRecord') {
        previewSDK.service.invokeCallback('ok', event, callbackId, {});
        return
      }

      if (cname === 'stopRecord') {
        previewSDK.service.invokeCallback('ok', event, callbackId, {
          tempVideoPath: res.tempVideoPath,
          tempThumbPath: res.tempThumbPath
        });
      }
    });
    if (cname === 'cameraFrameListenerStart') {
      electron.ipcMain.on(`cameraContextInvoke-${invokeId}`, (e, res) => {
        previewSDK.service.invokeCallback('ok', event, callbackId, {});
        previewSDK.service.send('customEvent', {
          event: 'cameraFrameListener',
          params: {
            url: res.url,
            width: res.width,
            height: res.height
          }
        });
      });
    }
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const cameraHandler = {
  cameraContextInvoke
};

const UDPSockets = {};

const emitSocketEvent$1 = (previewSDK, socketId, type, payload) => {
  previewSDK.service.send('customEvent', {
    event: 'onUDPEvent',
    params: { socketId, type, payload }
  });
};

const createUDPSocket = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { socketId } = params;
    const server = dgram__default["default"].createSocket('udp4');
    UDPSockets[socketId] = server;
    server.on('error', (err) => {
      console.error(err);
      emitSocketEvent$1(previewSDK, socketId, 'onError', { errMsg: err.toString() });
      server.close();
    });

    server.on('message', (msg, rinfo) => {
      emitSocketEvent$1(previewSDK, socketId, 'onMessage', {
        message: msg.toString('base64'),
        ...rinfo
      });
    });

    server.on('listening', () => {
      emitSocketEvent$1(previewSDK, socketId, 'onListening', {});
    });

    server.on('close', () => {
      emitSocketEvent$1(previewSDK, socketId, 'onClose', {});
      delete UDPSockets[socketId];
    });
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const bindUDPPort = (event, callbackId, params, ipcEvent, previewSDK) => {
  const { socketId } = params;
  portfinder__default["default"].getPort({ port: params.port || parseInt(Math.random() * 60000 + 5535) }, (err, port) => {
    try {
      if (err) throw err
      if (params.port && params.port !== port) {
        throw new Error(`端口${port}已被占用`)
      }
      if (!UDPSockets[socketId]) throw new Error('socket实例已经注销')
      UDPSockets[socketId].bind(port);
      UDPSockets[socketId].isBindded = true;
      previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, { port });
    } catch (error) {
      emitSocketEvent$1(previewSDK, socketId, 'onError', { errMsg: error.toString() });
      previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, { errMsg: error.toString() });
    }
  });
};

const sendUDPMessage = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    // TODO: 域名有效性校验
    const { socketId, message, offset, length, port, address } = params;
    const socket = UDPSockets[socketId];
    if (!socket) throw new Error(`UDPSocket(socketId = ${socketId}) does not exist`)
    if (!socket.isBindded) throw new Error(`UDPSocket(socketId = ${socketId}) is not bindded`)
    const msg = Buffer.from(message, 'base64');
    socket.send(msg, offset || 0, length || msg.byteLength, port, address);
  } catch (error) {
    emitSocketEvent$1(previewSDK, params.socketId, 'onError', { errMsg: error.toString() });
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const connetUDPSocket = (event, callbackId, params, ipcEvent, previewSDK) => {
  // 微信基础库中这个也是个空方法，不知为何也要加到文档里面，同微信实现
  // try {
  //   const { socketId, port, address } = params
  //   const socket = UDPSockets[socketId]
  //   socket.connect(port, address)
  // } catch (error) {
  //   emitSocketEvent(params.socketId, 'onError', { errMsg: error.toString() })
  //   previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() })
  // }
};

const closeUDPSocket = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { socketId } = params;
    const socket = UDPSockets[socketId];
    socket.close();
  } catch (error) {
    emitSocketEvent$1(previewSDK, params.socketId, 'onError', { errMsg: error.toString() });
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const setUDPTTL = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { socketId } = params;
    const socket = UDPSockets[socketId];
    socket.setTTL(params.ttl);
  } catch (error) {
    emitSocketEvent$1(previewSDK, params.socketId, 'onError', { errMsg: error.toString() });
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const networkHandle = {
  createUDPSocket,
  bindUDPPort,
  sendUDPMessage,
  connetUDPSocket,
  closeUDPSocket,
  setUDPTTL
};

const TCPSockets = {};

const emitSocketEvent = (previewSDK, socketId, type, payload) => {
  previewSDK.service.send('customEvent', {
    event: 'onTCPEvent',
    params: { socketId, type, payload }
  });
};

const _getAddressErrMsg = (address, port) => {
  return `connect fail: invalid address ${address}:${port}`
};
const _checkBlockPortList = (previewSDK, socketId, address, port) => {
  const blockPortList = [1433, 1521, 1719, 1720, 1723, 2049, 2375, 3128, 3306, 3389, 3659, 4045, 5060, 5061, 5432, 5984, 6379, 6000, 6566, 7001, 7002, 8443, 8888, 9200, 9300, 10051, 10080, 11211, 27017, 27018, 27019];
  const errInfo = { errMsg: _getAddressErrMsg(address, port), errCode: -4 };
  const isForbidPort = blockPortList.some(item => item === port);

  if (isForbidPort || (port >= 0 && port <= 1024) || (port >= 8000 && port <= 8100)) {
    emitSocketEvent(previewSDK, socketId, 'onError', errInfo);
    return true
  }

  return false
};

const _checkBlockAddress = (previewSDK, socketId, remoteAddress, remotePort) => {
  const address = ip__default["default"].address();
  const errInfo = { errMsg: _getAddressErrMsg(remoteAddress, remotePort), errCode: -4 };
  const reg = /(\w+\.){3}/;
  const matchResult = address.match(reg);
  const addressFrag = matchResult.length && matchResult[0];
  if (remoteAddress === address) {
    emitSocketEvent(previewSDK, socketId, 'onError', errInfo);
    return true
  }

  if (!remoteAddress.startsWith(addressFrag)) {
    emitSocketEvent(previewSDK, socketId, 'onError', errInfo);
    return true
  }
};

const _getSocketInfo = (socketId, data) => {
  if (!TCPSockets[socketId]) return
  const client = TCPSockets[socketId];
  const result = {
    localInfo: client.address(),
    remoteInfo: {
      address: client.remoteAddress,
      port: client.remotePort,
      family: client.remoteFamily,
      size: data.byteLength
    }

  };
  return result
};

const _initListeners = (previewSDK, socketId) => {
  if (!TCPSockets[socketId]) return
  const client = TCPSockets[socketId];
  client.on('connect', () => {
    emitSocketEvent(previewSDK, socketId, 'onConnect');
  });

  client.on('error', (err) => {
    console.error(err);
    emitSocketEvent(previewSDK, socketId, 'onError', { socketId, errMsg: err.toString(), errCode: -2 });
  });
  client.on('close', () => {
    emitSocketEvent(previewSDK, socketId, 'onClose');
  });
  client.on('data', (buffer) => {
    const rinfo = _getSocketInfo(socketId, buffer);
    emitSocketEvent(previewSDK, socketId, 'onMessage', { message: buffer.toString('base64'), socketId, ...rinfo });
  });
};

const createTCPSocket = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { socketId } = params;
    const client = new net__default["default"].Socket();
    TCPSockets[socketId] = client;
    _initListeners(previewSDK, socketId);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const connectTCPSocket = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { socketId, address, port } = params;
    if (_checkBlockPortList(previewSDK, socketId, address, port)) return
    if (_checkBlockAddress(previewSDK, socketId, address, port)) return
    const client = TCPSockets[socketId];
    client.connect(port, address);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const sendTCPMessage = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { socketId, message } = params;
    const client = TCPSockets[socketId];
    const msg = Buffer.from(message, 'base64');
    client.write(msg);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};

const closeTCPSocket = (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    const { socketId } = params;
    const client = TCPSockets[socketId];
    client.end();
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};
var tcpSocket = {
  createTCPSocket,
  connectTCPSocket,
  sendTCPMessage,
  closeTCPSocket
};

// 把 service 发送来的事件转发给 view
function sendEventToView (event, callbackId, payload, ipcEvent, previewSDK) {
  const eventId = uuid.v4();
  if (!previewSDK.route.currentView) {
    return previewSDK.service.invokeCallback('fail', event, ipcEvent, { errMsg: '找不到当前的 view' })
  }
  previewSDK.route.currentView.event.once(`onWebRTCEvent-${eventId}`, (res) => {
    if (res.errMsg) {
      return previewSDK.service.invokeCallbackSync('fail', event, ipcEvent, res)
    }
    previewSDK.service.invokeCallbackSync('ok', event, ipcEvent, res);
  });
  previewSDK.route.currentView.send('onWebRTCEvent', {
    eventName: event,
    eventId,
    method: payload.method,
    params: payload.params
  });
}

var webRTC = {
  mediaDevices: sendEventToView,
  mediaStream: sendEventToView,
  mediaRecorder: sendEventToView,
  rtcPeerConnection: sendEventToView
};

/**
 *
 * @param {(params: any, result: {ok: (res?:any) => void, fail:(error?:any) => void, previewSDK: import('#/index').default })=>void} serviceFn
 * @returns
 */
const createServiceApi = (serviceFn) => {
  return (event, callbackId, params, ipcEvent, previewSDK) => {
    const fail = (error) =>
      previewSDK.service.invokeCallback('fail', event, callbackId, {
        errMsg: error.toString(),
      });
    const ok = (res = {}) =>
      previewSDK.service.invokeCallback('ok', event, callbackId, res);

    try {
      serviceFn.call(null, params, { ok, fail, previewSDK });
    } catch (error) {
      fail(error);
    }
  }
};

const invokeRecorderManager = (previewSDK, params, eventCallback) => {
  const invokeId = uuid.v4();
  params.invokeId = invokeId;
  previewSDK.route.currentView.send('recorderManager', params);
  electron.ipcMain.on(`onRecorderManager-${invokeId}`, (e, res) => {
    eventCallback && eventCallback(res);
    previewSDK.service.send('customEvent', {
      event: 'onRecorderManager',
      params: res,
    });
  });
};

const recorderManager = async (event, callbackId, params, ipcEvent, previewSDK) => {
  try {
    invokeRecorderManager(previewSDK, params);
  } catch (error) {
    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg: error.toString() });
  }
};
const startRecord = createServiceApi((params, result) => {
  const { previewSDK } = result;
  invokeRecorderManager(
    previewSDK,
    {
      method: 'start',
      data: { duration: 60000 },
    },
    (res) => {
      const { data, method } = res || {};
      if (method === 'onStop') {
        result.ok({ tempFilePath: data.tempFilePath });
      }
    }
  );
});

const stopRecord = createServiceApi((params, result) => {
  const { previewSDK } = result;
  previewSDK.route.currentView.send('recorderManager', {
    method: 'stop',
  });
  result.ok();
});

const recorder = {
  recorderManager,
  startRecord,
  stopRecord,
};

const wechatAdaptMsg =
  '当前接口需要适配后支持微信用户登录功能，请参考： https://www.finclip.com/mop/document/introduce/functionDescription/wx-login.html';

const returnErr = (event, callbackId, payload, ipcEvent, previewSDK) => {
  return previewSDK.service.invokeCallback('fail', event, callbackId, {
    wechatAdaptMsg,
  })
};

var wechat = {
  login: returnErr,
  getUserProfile: returnErr,
};

var buttonOpenType = {
  buttonOpenTypeEvent(event, callbackId, payload, ipcEvent, previewSDK) {
    const { method, params } = payload || {};

    const errMsg = method === 'getPhoneNumber' ? wechatAdaptMsg : '未实现该 api';

    previewSDK.service.invokeCallback('fail', event, callbackId, { errMsg });
  },
};

/**
 * @type {Record<string, (event: string, callbackId: string|number, params: any, ipcEvent:Electron.IpcMainEvent, previewSDK:import('#/index').default) => void>}
 */
const serviceInvokeHandler = {
  ...basicHandler,
  ...deviceHandler,
  ...fileHandler,
  ...inputHandler,
  ...canvasHandler,
  ...localtionHandler,
  ...networkHandle$1,
  ...interfaceApis,
  ...media,
  ...navigationBar,
  ...storage,
  ...shareMenu,
  ...cameraHandler,
  ...networkHandle,
  ...tcpSocket,
  ...otherAPIHandler,
  ...webRTC,
  ...recorder,
  ...wechat,
  ...buttonOpenType,
};

// 这个模块用于处理 header Set-cookie 字段被 chrome 默认安全策略拦截，导致返回值中没有的问题

// KEY: requestId value: url
const requestMap = {};

// key: url value: set-cookie 等
const requestExtraInfoMap = {};

global.requestMap = requestMap;
global.requestExtraInfoMap = requestExtraInfoMap;

// 目前实现机制可能的风险： 基于 request url 来判断请求，如果连续请求同一个 url，可能导致返回的 set-cookie 对应不上的问题

const setCookieHandler = (event, method, params, resourceType) => {
  if (method === 'Network.requestWillBeSent') {
    const url = params.request.url;
    const requestId = params.requestId;
    requestMap[requestId] = url;
  }

  if (method === 'Network.loadingFailed' || method === 'Network.loadingFinished') {
    setTimeout(() => {
      const url = requestMap[params.requestId];
      delete requestMap[params.requestId];
      delete requestExtraInfoMap[url];
    }, 1000);
  }

  if (method === 'Network.responseReceivedExtraInfo') {
    if (params.headers && (params.headers['Set-Cookie'] || params.headers['set-cookie'])) {
      const setCookie = params.headers['Set-Cookie'] || params.headers['set-cookie'];
      const url = requestMap[params.requestId];
      if (url) {
        requestExtraInfoMap[url] = { setCookie };
      }
    }
  }
};

electron.ipcMain.handle('getRequestExtraHeader', (event, params) => {
  const url = params.url || '';
  return requestExtraInfoMap[url] || {}
});

class Service {
  /**
   * @param {import('#/index').default} app 
   */
  constructor (app) {
    this.app = app;
    this.isReady = false;
    this.webviewId = uuid.v4();
    this.webcontent = null;
    this.status = null;
    this.customSyncApiObj = {};
    this.create();
    
    /**
     * @type {Electron.WebContents}
     */
    this.webcontent;
    /**
     * @type {EventManager}
     */
    this.event;
    /**
     * @type {Record<string, boolean>}
     */
    this.customSyncApiObj;
  }

  create () {
    this.event = new EventManager(this.app, this.webviewId, null, 'service');
    this.event.on('invoke', (data, ipcEvent) => {
      this.handleInvoke(data.event, data.params, data.callbackId, ipcEvent);
    });
    this.event.on('publish', (data) => {
      this.handlePublish(data.event, data.params, data.webviewIds);
    });
    this.event.on('extApiRegisted', (data) => {
      this.app._emitSdkEvent('extApiRegisted', data);
    });
    this.app._emitSdkEvent('CREATE_WEBVIEW', {
      src: 'service.html',
      webviewId: this.webviewId,
      type: 'service'
    });
    this.isReady = true;

    this.event.onRegister(() => {
      try {
        this.webcontent = electron.webContents.fromId(this.event.id);
        this.webcontent.debugger.attach('1.3');
        this.webcontent.debugger.sendCommand('Network.enable');
        this.webcontent.debugger.on('message', setCookieHandler);
      } catch (err) {
        console.error(err);
      }
    });
  }

  reload () {
    if (this.event.id) {
      this.app._emitSdkEvent('SERVICE_RELOAD');
      const webcontent = electron.webContents.fromId(this.event.id);
      webcontent && webcontent.executeJavaScript('window.location.reload()');
      this.customSyncApiObj = {};
    }
  }

  destroy() {
    // 关闭音乐播放
    this.app._emitSdkEvent('stopVoice');
    this.app._emitSdkEvent('SERVICE_RELOAD');
    this.app._emitSdkEvent('DESTROY_WEBVIEW', {
      webviewId: this.webviewId,
    });
    this.event.destroy();
    if (this.webcontent && !this.webcontent.isDestroyed()) {
      this.webcontent.debugger.off('message', setCookieHandler);
      this.webcontent.debugger.detach();
    }
    this.customSyncApiObj = {};
  }

  send (event, data) {
    this.event.send(event, data);
  }

  registerCustomSyncApi(apiName) {
    this.customSyncApiObj[apiName] = true;
  }

  invokeCallback (status, event, callbackId, params = {}) {
    const res = {
      ...params,
      errMsg: `${event}:${status}${params.errMsg ? ' ' + params.errMsg : ''}`
    };
    this.event.send('invokeCallback', {
      callbackId,
      params: res
    });
  }

  invokeCallbackSync (status, event, ipcEvent, params = {}) {
    const res = {
      ...params,
      errMsg: `${event}:${status}${params.errMsg ? ' ' + params.errMsg : ''}`
    };
    ipcEvent.returnValue = res;
  }

  onReady(params) {
    const subpackages = params.subpackages || params.subPackages;
    if (Array.isArray(subpackages)) {
      subpackages.forEach(i => {
        i.pageFullPath = i.pages.map(path => i.root + '/' + path);
      });
    }
    this.app.appConfig = params;

    const ideEntryConfig = this.getCustomEntryConf();
    let queryObj = {};
    if (ideEntryConfig.param) {
      queryObj = parseQuery(ideEntryConfig.param);
    }
    this.app.sendToService('onServiceReadyDone', {
      path: ideEntryConfig.path,
      query: queryObj,
    });
    this.app.route.onReady();
  }

  onRoute (type, path, openType, isTab) {
    this.app.route[type](path, openType, isTab);
  }

  async handleRouteEvent(event, params, callbackId, ipcEvent) {
    const {
      url,
      delta
    } = params;
    // navigateBack 不需判断 url
    const isExist = event === 'navigateBack' ? true : this.app.route.checkPathIsExist(url);

    if (!isExist) {
      this.invokeCallback('fail', event, callbackId, { errMsg: `${event} url is not found` });
      return
    }

    switch (event) {
      case 'navigateTo':
      case 'redirectTo':
        const isTab = this.app.route.tabs.find(item => {
          return item.path === url
        });
        if (isTab) {
          this.invokeCallback('fail', event, callbackId, { errMsg: '不允许跳转 tabbar 页面, 请使用 switchTab' });
        } else {
          this.onRoute(event, url, event);
          this.invokeCallback('ok', event, callbackId, {});
        }
        break
      case 'switchTab':
      case 'reLaunch':
        this.onRoute(event, url, event, event === 'switchTab');
        this.invokeCallback('ok', event, callbackId, {});
        break
      case 'navigateBack':
        if (this.app.route.stack.length === 1) {
          this.invokeCallback('fail', event, callbackId, { errMsg: 'cannot navigate back at first page.' });
        } else {
          this.onRoute(event, delta, event);
          this.invokeCallback('ok', event, callbackId, {});
        }
        break
    }
  }

  handleInvoke (event, params, callbackId, ipcEvent) {
    if (/^(navigateTo|redirectTo|switchTab|reLaunch|navigateBack)$/.test(event)) {
      this.handleRouteEvent(event, params, callbackId, ipcEvent);
    }

    try {
      if (this.app.appStore.data.mockApi) {
        const mockApi = this.app.appStore.data.mockApi;
        if (mockApi[event] && mockApi[event].mockType === 'API') {
          console.log('MockApi', event, mockApi[event].returnValue);
          if (mockApi[event].returnType === 'Success') {
            if (this.customSyncApiObj[event]) {
              console.log('invokeCallbackSync', 'ok', event, ipcEvent, JSON.parse(mockApi[event].returnValue));
              this.invokeCallbackSync('ok', event, ipcEvent, JSON.parse(mockApi[event].returnValue));
            } else {
              this.invokeCallback('ok', event, callbackId, JSON.parse(mockApi[event].returnValue));
            }
            return
          }
          if (mockApi[event].returnType === 'Fail') {
            if (this.customSyncApiObj[event]) {
              this.invokeCallbackSync('fail', event, ipcEvent, JSON.parse(mockApi[event].returnValue));
            } else {
              this.invokeCallback('fail', event, callbackId, JSON.parse(mockApi[event].returnValue));
            }
            return
          }
        }
      }
    } catch (error) {
      console.log('mockApiError', error);
    }

    if (typeof serviceInvokeHandler[event] === 'function') {
      serviceInvokeHandler[event](event, callbackId, params, ipcEvent, this.app);
    } else {
      if (event.endsWith('Sync') || this.customSyncApiObj[event]) {
        this.invokeCallbackSync('fail', event, ipcEvent, { errMsg: '未实现该 api' });
      } else {
        this.invokeCallback('fail', event, callbackId, { errMsg: '未实现该 api' });
      }
    }
  }

  getCustomEntryConf() {
    try {
      const compileConfig = this.app.appStore.data.compileConfig[this.app.id || this.app.name];
      const res = {};
      if (compileConfig.current) {
        res.path = compileConfig.current.page;
        res.param = compileConfig.current.param;
      }
      return res
    } catch (error) {
      return {}
    }
  }

  launchInitPages(ftConfig, reLaunchOpts = {}) {
    const customEntryConf = reLaunchOpts.path ? reLaunchOpts : this.getCustomEntryConf();
    const path = customEntryConf.path || ftConfig.entryPagePath || ftConfig.pages[0];
    const tabBar = ftConfig.tabBar;
    const hasTab = tabBar && Array.isArray(tabBar.list);
    let isEntryTabPage = false;
    if (hasTab) {
      tabBar.list.forEach((item, i) => {
        if (item.pagePath === path) {
          isEntryTabPage = true;
          this.onRoute('launch', `${item.pagePath}.html${customEntryConf.param ? '?' + customEntryConf.param : ''}`, reLaunchOpts.path ? 'reLaunch' : 'appLaunch', true);
        } else {
          setTimeout(() => {
            this.onRoute('preload', `${item.pagePath}.html${customEntryConf.param ? '?' + customEntryConf.param : ''}`, 'switchTab', true);
          }, 50 * i);
        }
      });
    }
    if (!isEntryTabPage) {
      this.onRoute('launch', `${path}.html${customEntryConf.param ? '?' + customEntryConf.param : ''}`, reLaunchOpts.path ? 'reLaunch' : 'appLaunch', false);
    }
  }

  registerCustomSyncApis() {
    if (this.app.finclipConfig.extApi) {
      try {
        this.app.finclipConfig.extApi.forEach(item => {
          if (item.sync) this.customSyncApiObj[item.name] = true;
          if (Object.keys(this.customSyncApiObj).length) {
            this.send('registerCustomSyncApis', this.customSyncApiObj);
          }
        });
      } catch (err) {
        console.error(err);
      }
    }
  }

  handlePublish (event, params, webviewIds) {
    switch (event) {
      case 'custom_event_serviceStart':
        this.app._emitSdkEvent('SERVICE_START', params);
        this.app.ftConfig = params;
        this.launchInitPages(this.app.ftConfig);
        this.status = 'start';
        setTimeout(() => {
          if (this.status === 'start') {
            this.app._emitSdkEvent('APPLET_SERVICE_LOAD_FAIL');
          }
        }, 5000);
        this.registerCustomSyncApis();
        break
      case 'custom_event_serviceReady':
        this.status = 'ready';
        this.onReady(params);
        this.app._emitSdkEvent('SERVICE_READY', params);
        break
      default:
        if (event.startsWith('custom_event_')) {
          this.app.sendToViews(event, params, webviewIds);
        }
        break
    }
  }
}

var BuildEvent = {
  START: 'build_event_start',
  BUILD_DATA: 'build_event_data',
  BUILD_CLOSE: 'build_event_close',
  BUILD_SUCCESS: 'build_event_success',
  BUILD_ERROR: 'build_event_error'
};

const getCurTimeString = () => {
  const d = new Date();
  return `${d.getFullYear()}-${d.getMonth() + 1}-${d.getDate()} ${d.getHours()}:${d.getMinutes()}:${d.getSeconds()}`
};

const watchConfig$1 = {
  usePolling: false,
  ignoreInitial: true,
  persistent: true,
  awaitWriteFinish: {
    stabilityThreshold: 1000,
    pollInterval: 100,
  },
  ignored: /.DS_Store|\.git/

};
class AppletBuilder {
  constructor(args) {
    this.process = null;
    this._pendingProcess = null;
    this._preloadProcess = null;
    // this.prepareProcess()
  }

  prepareProcess(useExtraProcess) {
    // if (this._preloadProcess && !this._preloadProcess.killed) return this._preloadProcess

    // if (this._pendingProcess) {
    //   return new Promise((resolve, reject) => {
    //     this._pendingProcess.on('spawn', () => {
    //       resolve(this._pendingProcess)
    //     })
    //   })
    // }

    if (useExtraProcess) {
      const process = ChildProcess__default["default"].fork(path__default["default"].join(__dirname, 'build.js'), {
        silent: true,
        env: {
          LOG_PATH: path__default["default"].join(electron.app.getPath('userData'), 'log')
        }
      });
      process.on('message', message => {
        const [type, msg] = message;
        switch (type) {
          case 'BUILD_SUCCESS':
          case 'BUILD_ERROR':
          case 'BUILD_CLOSE':
            process.kill(2);
        }
      });
    }

    if (this.process) {
      this.process.kill(2);
    }
    this.process = ChildProcess__default["default"].fork(path__default["default"].join(__dirname, 'build.js'), {
      silent: true,
      env: {
        LOG_PATH: path__default["default"].join(electron.app.getPath('userData'), 'log')
      }
    });
    this.process.stdout.on('data', (chunk) => {
      console.log('Builder process:', chunk.toString());
    });
    // this.process.on('close', () => {
    //   this.process = null
    // })

    return this.process

    // return new Promise((resolve, reject) => {
    //   this._pendingProcess.on('spawn', () => {
    //     this._preloadProcess = this._pendingProcess
    //     setTimeout(() => {
    //       this._pendingProcess = null
    //     }, 0)
    //     resolve(this._pendingProcess)
    //   })
    //   this._pendingProcess.on('error', (err) => {
    //     reject(err)
    //   })
    // })
  }

  // AppletBuilder 的 event 和 watcher 在内部维护，每次 start 时清空绑定
  stop() {
    this.process && this.process.kill(2);
    // this.prepareProcess()
    if (this.watcher) {
      this.watcher.close();
      this.watcher = null;
    }
  }

  async handlePlugins(event, plugins, localDevPluginPath) {
    if (!Object.keys(plugins).length) return
    event.sender.send(BuildEvent.BUILD_DATA, {
      data: '开始准备插件',
    });

    const userDataPath = electron.app.getPath('userData');
    const tmpPath = path__default["default"].join(userDataPath, './tmp');

    // TODO 超时判断
    const pluginRes = [];

    try {
      await Promise.all(
        Object.keys(plugins).map(name => {
          if (plugins[name].version === 'dev') {
            return new Promise((resolve, reject) => {
              pluginRes.push({
                dir: localDevPluginPath,
                id: plugins[name].provider || plugins[name].id,
                version: plugins[name].version,
                isEncrypted: false,
              });
              resolve(true);
            })
          } else {
            return new Promise((resolve, reject) => {
              event.sender.send(BuildEvent.BUILD_DATA, {
                data: `开始获取插件信息pluginId: ${
                  plugins[name].provider || plugins[name].id
                }, version: ${plugins[name].version}`,
              });
              axios__default["default"]
                .get(
                  `${global.appConfig.pluginBaseURL}/api/v2/mop/mop-applet-build-manager/plugin/ide/version/info`,
                  {
                    params: {
                      pluginId: plugins[name].provider || plugins[name].id,
                      version: plugins[name].version,
                    },
                  }
                )
                .then(res => {
                  if (
                    res.status === 200 &&
                    res.data &&
                    res.data.errcode === 'OK'
                  ) {
                    axios__default["default"]({
                      method: 'get',
                      url: `${global.appConfig.pluginBaseURL}${res.data.data.codePackageUrl}`,
                      responseType: 'arraybuffer',
                    }).then(dwRes => {
                      const destZipPath = path__default["default"].join(tmpPath, `${name}.zip`);
                      const destDirPath = path__default["default"].join(tmpPath, name);
                      fs__default["default"].writeFileSync(destZipPath, dwRes.data);
                      event.sender.send(BuildEvent.BUILD_DATA, {
                        data: `下载插件 pluginId: ${
                          plugins[name].provider || plugins[name].id
                        }, version: ${plugins[name].version} 完成`,
                      });

                      const unzipStream = fs__default["default"].createReadStream(destZipPath);
                      unzipStream.pipe(unzipper__default["default"].Extract({ path: destDirPath }))
                        .on('entry', entry => entry.autodrain())
                        .promise().then(() => {
                          event.sender.send(BuildEvent.BUILD_DATA, {
                            data: `解压插件完成 pluginId: ${
                              plugins[name].provider || plugins[name].id
                            }, version: ${plugins[name].version}`,
                          });
                          pluginRes.push({
                            dir: destDirPath,
                            id: plugins[name].provider || plugins[name].id,
                            version: plugins[name].version,
                            isEncrypted: true,
                          });
                          resolve(destZipPath);
                        }, e => {
                          event.sender.send(BuildEvent.BUILD_ERROR, {
                            data: `解压插件失败 pluginId: ${
                              plugins[name].provider || plugins[name].id
                            }, version: ${plugins[name].version}`,
                          });
                          reject(new Error('解压插件失败'));
                        });
                    });
                  } else {
                    event.sender.send(BuildEvent.BUILD_DATA, {
                      data: `获取插件信息失败 pluginId: ${
                        plugins[name].provider || plugins[name].id
                      }, version: ${plugins[name].version}`,
                    });
                    reject(new Error('获取插件信息失败'));
                  }
                })
                .catch(err => {
                  event.sender.send(BuildEvent.BUILD_DATA, {
                    data: `插件不存在，请检查插件ID或版本号 pluginId: ${
                      plugins[name].provider || plugins[name].id
                    }, version: ${plugins[name].version}`,
                  });
                  reject(err);
                });
            })
          }
        })
      );
    } catch (error) {
      console.error('handlePlugins error', error);
    }

    event.sender.send(BuildEvent.BUILD_DATA, {
      data: '准备插件结束',
    });

    return pluginRes
  }

  async start(event, devMode, srcDir, destDir, option, useExtraProcess) {
    let firstBuild = true;
    event.sender.send(BuildEvent.START, {});
    const userDataPath = electron.app.getPath('userData');
    const destPath = destDir || path__default["default"].join(userDataPath, './dest');

    event.sender.send(BuildEvent.BUILD_DATA, {
      data: `${getCurTimeString()} 开始删除旧的编译文件`,
    });
    if (fs__default["default"].existsSync(destPath)) {
      fs__default["default"].rmSync(destPath, { recursive: true, force: true });
    }
    event.sender.send(BuildEvent.BUILD_DATA, {
      data: `${getCurTimeString()} 删除旧的编译文件结束`,
    });
    fs__default["default"].ensureDirSync(destPath);

    const tmpPath = path__default["default"].join(userDataPath, './tmp');
    srcDir = srcDir.replace(/\\$|\$|\/$/, '');
    if (!useExtraProcess && option.watch) {
      this.appJsonWatch(srcDir, option);
    }

    if (option.plugins) {
      const plugins = await this.handlePlugins(
        event,
        option.plugins,
        option.pluginRoot && option.pluginRoot.replace(/\\$|\$|\/$/, '')
      );
      option.plugins = plugins;
    }

    const process = await this.prepareProcess(useExtraProcess);

    return new Promise(resolve => {
      process.send({
        type: 1,
        devMode,
        srcDir: srcDir,
        destPath,
        tmpPath,
        option,
      });
      process.on('message', async message => {
        const [type, msg] = message;
        switch (type) {
          case 'BUILD_START':
            buildLogger.info(msg);
            // event.sender.send(BuildEvent.BUILD_DATA, {
            //   data: msg,
            // })
            break
          case 'BUILD_DATA':
            buildLogger.info(msg);
            event.sender.send(BuildEvent.BUILD_DATA, {
              data: msg,
            });
            break
          case 'BUILD_CLOSE':
            buildLogger.info(msg);
            event.sender.send(BuildEvent.BUILD_CLOSE, {
              data: msg,
            });
            resolve();
            break
          case 'BUILD_ERROR':
            buildLogger.error(msg);
            console.log('BUILD_ERROR', msg);
            event.sender.send(BuildEvent.BUILD_ERROR, {
              data: '编译失败',
              info: msg
            });
            resolve();
            break
          case 'BUILD_SUCCESS':
            if (this.watcher) {
              this.watcher.close();
              this.watcher = null;
            }
            await this.afterBuild(destPath);
            buildLogger.info(`BUILD_SUCCESS ${destPath}`);
            event.sender.send(BuildEvent.BUILD_SUCCESS, {
              data: destPath,
            });
            if (!firstBuild && msg && msg.isWatch) {
              option.onWatcherBuild();
            } else {
              firstBuild = false;
              resolve();
            }
            break
          case 'BUILD_RESET':
            option.reloadBuild();
            break
        }
      });
    })
  }

  async afterBuild(destPath) {
    fs__default["default"].copySync(path__default["default"].join(destPath, 'package'), destPath);
    try {
      await fs__default["default"].copySync(path__default["default"].join(destPath, 'package'), destPath);
      const pageConfig = await fs__default["default"].readJSONSync(path__default["default"].join(destPath, 'packageConfig.json')) || [];
      for (let index = 0; index < pageConfig.length; index++) {
        const element = pageConfig[index];
        if (element.name !== '__APP__') {
          await mergeDir(path__default["default"].join(destPath, element.name), path__default["default"].join(destPath, '__APP__'));
        }
      }
    } catch (error) {
      console.error('afterBuild error', error);
    }
  }

  /**
   * 在编译未成功前监听文件变动 触发重新编译
   * @param {*} srcDir
   * @param {*} option
   */
  appJsonWatch(srcDir, option) {
    let time = null;
    if (this.watcher) {
      this.watcher.close();
      this.watcher = null;
    }
    const watcher = chokidar__default["default"].watch(srcDir, watchConfig$1);
    watcher.on('all', async (e, path) => {
      if (!time || new Date().getTime() - time > 3000) {
        option.reloadBuild();
      }
      time = new Date().getTime();
    });
    this.watcher = watcher;
  }
}

var builderV2 = new AppletBuilder();

let latitude = 22.93772;
let longitude = 22.93772;

electron.ipcMain.on('setLocationInfo', (e, payload) => {
  latitude = payload.latitude || 22.93772;
  longitude = payload.longitude || 22.93772;
});

const downloadFile = async (params, webviewId, htmlId, callback = () => {}, previewSDK) => {
  const { url, header, timeout, filePath } = params;
  try {
    if (timeout && typeof timeout !== 'number') throw new Error('timeout must be number')
    if (!validateUrl(url, '')) ;

    let savePath;
    if (filePath) {
      if (fs__default["default"].existsSync(filePath)) throw new Error('filePath 文件已存在')
      fs__default["default"].ensureDirSync(path__default["default"].dirname(filePath));
      savePath = filePath;
    }
    const controller = new AbortController__default["default"]();
    let wStream = null;

    const reqOpts = {
      headers: header || {},
      signal: controller.signal,
      savePath
    };

    if (timeout && timeout > 0) {
      setTimeout(() => {
        controller.abort();
        wStream && wStream.end(() => {
          fs__default["default"].unlinkSync(savePath);
        });
        callback('fail', { errMsg: 'timeout' });
      }, timeout);
    }

    const res = await _fetch__default["default"](url, reqOpts);

    const fileName = `${uuid.v4()}${getFileExt(
      url,
      res.headers.get('content-type'),
      res.headers.get('content-disposition')
    )}`;
    if(!filePath) {
      savePath = path__default["default"].join(previewSDK.fileCachePath, 'temp', fileName);
    }
    wStream = fs__default["default"].createWriteStream(savePath);

    const resHeader = res.headers.raw();
    const farmatedHeader = formatHeader(resHeader);
    previewSDK.service.send('customEvent', {
      event: 'onTaskHeadersReceived',
      params: { header: farmatedHeader }
    });

    wStream.on('error', error => callback('fail', { errMsg: error.toString() }));
    wStream.on('finish', () => {
      const suceesRes = {
        statusCode: res.status,
        cookies: resHeader['set-cookie'] || [],
        header: farmatedHeader,
        profile: {},
      };
      if (filePath) {
        suceesRes.filePath = savePath;
      } else {
        suceesRes.tempFilePath = `finfile://temp/${fileName}`;
      }
      callback('ok', suceesRes);
    });

    const totalSize = parseInt(res.headers.get('Content-Length'), 10);
    let bytesDone = 0;
    // TODO 对这个方法进行debounce 调用太频繁了
    res.body.on('data', (chunk) => {
      bytesDone += chunk.length;
      previewSDK.service.send('customEvent', {
        event: 'onTaskProgressUpdate',
        params: {
          progress: totalSize > 0 ? parseInt(bytesDone / totalSize * 100, 10) : NaN,
          totalBytesWritten: bytesDone,
          totalBytesExpectedToWrite: totalSize,
        }
      });
    });
    res.body.pipe(wStream);
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const chooseImage = async (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    const response = await electron.dialog.showOpenDialog(previewSDK.homeWin, {
      title: '打开',
      properties: ['openFile', 'multiSelections'],
      filters: [{ name: 'Images', extensions: ['jpg', 'jpeg', 'png', 'gif', 'svg'] }]
    });

    if (response.canceled) {
      callback('fail', { errMsg: 'cancel' });
      return
    }
    const result = {
      tempFilePaths: [],
      tempFiles: [],
    };
    if (response.filePaths && response.filePaths.length) {
      response.filePaths.forEach((file) => {
        const tempFile = saveFileToTempPath(file, previewSDK.fileCachePath);
        if (typeof tempFile === 'object') {
          result.tempFilePaths.push(tempFile.path);
          result.tempFiles.push(tempFile);
        }
      });
    }
    callback('ok', result);
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const getLocalImgData = (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  const { path: filePath = '' } = params;
  if (!filePath || !filePath.startsWith('finfile://')) {
    callback('fail', { errMsg: 'url 参数错误' });
    return
  }
  const pathname = path__default["default"].join(previewSDK.fileCachePath, decodeURI(filePath.replace('finfile://', '')));
  const isExists = fs__default["default"].existsSync(pathname);
  if (!isExists) return callback('fail', { errMsg: '文件不存在' })
  const buffer = fs__default["default"].readFileSync(pathname);
  callback('ok', {
    localData: `data:image/png;base64,${buffer.toString('base64')}`
  });
};

const openDocument = (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  const { filePath = '' } = params;
  if (!filePath || !filePath.startsWith('finfile://')) {
    callback('fail', { errMsg: 'url 参数错误' });
    return
  }
  const pathname = path__default["default"].join(previewSDK.fileCachePath, decodeURI(filePath.replace('finfile://', '')));
  const isExists = fs__default["default"].existsSync(pathname);
  if (!isExists) return callback('fail', { errMsg: '文件不存在' })
  electron.shell.openPath(pathname);
  callback('ok', {});
};

const setNavigationBarTitle = (params = {}, webviewId, htmlId, callback = () => { }, previewSDK) => {
  try {
    previewSDK.sendToHome('setNavigationBarTitle', { ...params, htmlId });
    callback('ok', {});
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const setStorage = async (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    const { key, data } = params;
    storageUtils.setStorage(key, data, previewSDK);
    callback('ok', {});
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const removeStorage = async (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    const { key } = params;
    storageUtils.removeStorage(key, previewSDK);
    callback('ok', {});
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const getStorage = async (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    const { key } = params;
    const data = storageUtils.getStorage(key, previewSDK);
    if (!data) callback('fail', { errMsg: 'data not found' });
    callback('ok', { data });
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const getStorageInfo = async (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    const info = storageUtils.getStorageInfo(previewSDK);
    callback('ok', info);
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const clearStorage = async (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    storageUtils.clearStorage(previewSDK);
    callback('ok', {});
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const close = (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  callback('ok', { errMsg: 'mock close success' });
};

const navigateToMiniProgram = (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  callback('ok', { errMsg: 'mock navigateToMiniProgram success' });
};

const navigateBackMiniProgram = (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  callback('ok', { errMsg: 'mock navigateBackMiniProgram success' });
};

// canGoBack 需要用到 document，在 preload webview.js 里处理
// export const canGoBack = (params = {}, webviewId, htmlId, callback = () => {}) => {}

const getLocation = (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  callback('ok', {
    accuracy: 65,
    horizontalAccuracy: 65,
    latitude,
    longitude,
    speed: -1,
    verticalAccuracy: 65,
  });
};

const chooseLocation = async (params = {}, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    const locationId = uuid.v4();
    previewSDK.sendToHome('chooseLocation', { ...params, locationId });
    const response = await new Promise((resolve, reject) => {
      previewSDK.homeEventManager.once(`chooseLocation-${locationId}`, (res) => {
        if (res.cancel) {
          reject('cancel');
        } else {
          resolve(res);
        }
      });
    });
    callback('ok', response.data);
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const postMessage = ( params, webviewId, htmlId, callback = () => {}, previewSDK) => {
  try {
    previewSDK.service.send('customEvent', {
      event: 'onBindMessage',
      params: params,
      webviewId: webviewId,
    });
    callback('ok', {});
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

const getAppletInfo = (params, webviewId, htmlId, callback, previewSDK) => {
  try {
    previewSDK.sendToHome('getAppletInfo', params);
    previewSDK.homeEventManager.on('handleGetAppletInfo', (res) => {
      callback('ok', res);
    });
  } catch (error) {
    callback('fail', { errMsg: error.toString() });
  }
};

var apiInvokeHandler = {
  downloadFile,
  chooseImage,
  getLocalImgData,
  setNavigationBarTitle,
  getStorage,
  setStorage,
  removeStorage,
  clearStorage,
  getStorageInfo,
  openDocument,
  postMessage,
  getAppletInfo,
  close,
  navigateToMiniProgram,
  navigateBackMiniProgram,
  getLocation,
  chooseLocation
};

/**
 * 小程序中的 webview sdk 处理逻辑
 */
class HtmlWebview {
  constructor () {
    this.events = {};
  }
  /**
   * @param {import('#/index').default} app 
   */
  start(app) {
    this.app = app;
    this.app.on('webPublish', (data) => {
      this.handlePublish(data.event, data.params, data.webviewIds);
    });
  }

  init(app) {
    // mark: 仅允许初始化一次, 防止重复编译时重复绑定 ipcMain 事件
    if (!this.isInit) {
      this.isInit = true;
      this.start(app);
    } else {
      this.reload();
    }
    return this
  }

  reload() {
    Object.keys(this.events).forEach(htmlId => this.destroyWebviewById(htmlId));
    this.events = {};
  }

  beforeDestroy(htmlId) {
    if (this.events[htmlId] && this.events[htmlId].id) {
       this.events[htmlId].send('pageonunload', {
        webviewId: htmlId
      });
    }
  }

  destroyWebviewById(htmlId) {
    this.beforeDestroy(htmlId);
    const target = this.events[htmlId];
    delete this.events[htmlId];
    console.log('webview destroyWebview', htmlId);
    setTimeout(() => {
      this.app._emitSdkEvent('DESTROY_WEBVIEW', {
        webviewId: htmlId,
      });
      target && target.destroy();
    }, 100);
  }

  insertHTMLWebView(params = {}) {
    const { webviewId, htmlId } = params;
    const event = new EventManager(this.app, params.htmlId, null, 'webview');
    event.on('webInvoke', (data, ipcEvent) => {
      this.handleInvoke(data.event, data.params, data.callbackId, ipcEvent, webviewId, htmlId);
    });
    event.on('publish', (data, ipcEvent) => {
      this.app.sendToService('customEvent', data);
    });
    this.events[htmlId] = event;
    this.app.sendToService('customEvent', {
      event: 'insertHTMLWebView',
      params: params
    });
    this.app._emitSdkEvent('insertHTMLWebView', params);
  }

  updateHTMLWebView(params = {}) {
    this.app._emitSdkEvent('updateHTMLWebView', params);
  }

  removeHTMLWebView(params = {}) {
    const { htmlId } = params;
    this.clearHtmlWebview(htmlId);
    this.app._emitSdkEvent('removeHTMLWebView', params);
  }

  clearHtmlWebview(htmlId) {
    if (this.events[htmlId]) delete this.events[htmlId];
  }

  onRoute(type, path, openType, isTab) {
    this.app.route[type](path, openType, isTab);
  }

  handleInvoke(event, params, callbackId, ipcEvent, webviewId, htmlId) {
    let name = event;
    let arg = params;
    if (event === 'invokeMiniProgramAPI') {
      try {
        name = params.name;
        arg = params.arg;
      } catch (e) {

      }
    }

    if (/^(navigateTo|redirectTo|switchTab|reLaunch|navigateBack)$/.test(name)) {
      const {
        url,
        delta
      } = arg;

      const isExist = name === 'navigateBack' ? true : this.app.route.checkPathIsExist(url);

      if (!isExist) {
        this.invokeCallback('fail', event, callbackId, { errMsg: `${name} url is not found` }, htmlId);
        return
      }

      switch (name) {
        case 'navigateTo':
        case 'redirectTo':
          const isTab = this.app.route.tabs.find(item => {
            return item.path === url
          });
          if (isTab) {
            this.invokeCallback('fail', event, callbackId, { errMsg: '不允许跳转 tabbar 页面, 请使用 switchTab' }, htmlId);
          } else {
            this.invokeCallback('ok', event, callbackId, {}, htmlId);
            this.onRoute(name, url, name);
          }
          return
        case 'switchTab':
        case 'reLaunch':
          this.invokeCallback('ok', event, callbackId, {}, htmlId);
          this.onRoute(name, url, name, name === 'switchTab');
          return
        case 'navigateBack':
          this.invokeCallback('ok', event, callbackId, {}, htmlId);
          this.onRoute(name, delta, name);
          return
      }
    }

    if (/canNavigateBack/.test(name)) {
      const res = this.app.route.stack.length >= 2;
      this.invokeCallback('ok', event, callbackId, {
        canNavigateBack: res
      }, htmlId);
      return
    }

    if (/canGoBack/.test(name)) {
      this.app._emitSdkEvent('getWebviewInfo', { webviewId: htmlId });
      this.app.on('handleWebviewInfo', (data) => {
        this.invokeCallback('ok', event, callbackId, data, htmlId);
      });
      return
    }

    try {
      if (this.app.appStore.data && this.app.appStore.data.mockApi) {
        const mockApi = this.app.appStore.data.mockApi;
        if (mockApi[name] && mockApi[name].mockType === 'H5') {
          console.log('h5 MockApi', name, mockApi[name].returnValue);
          if (mockApi[name].returnType === 'Success') {
            this.invokeCallback('ok', event, callbackId, JSON.parse(mockApi[name].returnValue), htmlId);
            return
          }
          if (mockApi[name].returnType === 'Fail') {
            this.invokeCallback('fail', event, callbackId, JSON.parse(mockApi[name].returnValue), htmlId);
            return
          }
        }
      }
    } catch (error) {
      console.log('webview mockApiError', error);
    }
    // 获取已有的匹配的api进行调用
    if (typeof apiInvokeHandler[name] === 'function') {
      apiInvokeHandler[name](arg, webviewId, htmlId, (state, res) => {
        this.invokeCallback(state, event, callbackId, res, htmlId);
      }, this.app);
    } else {
      this.invokeCallback('fail', event, callbackId, { errMsg: '未实现该api' }, htmlId);
    }
  }

  handlePublish(event, params) {
    for (const id in this.events) {
      this.sendEvent(id, 'customEvent', {
        event: event,
        params: params
      });
    }
  }

  sendEvent(id, eventName, params, retry = 2) {
    const event = this.events[id];
    if (!event) return
    if (!event.id) {
      // webview 未加载完成时, event 的 id 会是空
      // send 事件会接收不到, 这里加个延时重试
      retry > 0 && setTimeout(() => {
        this.sendEvent(id, eventName, params, --retry);
      }, 500);
      return
    }
    event.send(eventName, params);
  }

  invokeCallback(statue, event, callbackId, params = {}, htmlId) {
    const eventIns = this.events[htmlId];
    if (!eventIns) return
    const res = {
      ...params,
      errMsg: `${event}:${statue}${params.errMsg ? ' ' + params.errMsg : ''}`,
    };
    eventIns.send('webInvokeCallback', {
      callbackId,
      params: res
    });

    eventIns.send('customEvent', {
      callbackId,
      params: res
    });
  }

  invokeCallbackSync(statue, event, ipcEvent, params = {}) {
    const res = {
      ...params,
      errMsg: `${event}:${statue}${params.errMsg ? ' ' + params.errMsg : ''}`,
    };
    ipcEvent.returnValue = res;
  }
}

var WebviewSDK = new HtmlWebview();

const algorithm = 'aes-192-cbc';
const passwordCrypto = 'LCJhbGciOiJIUzI1NiJ9.eyJmY2lkIjoiQGd1b3poaTpmaW5vZ2Vla3MuY2x1YiIsImlzcyI6IjRrN2llSGJlMDdPbHFCeUNQbHA3YlVPTjdpQkc2WHFnIiwiaWF0IjoxNTM1NjA3MDExfQ.rdyEBU6flep7O-62cQEgF1LjL-GV';
const key = crypto__default["default"].scryptSync(passwordCrypto, 'salt', 24);
const iv = Buffer.alloc(16, 0);

const encode = (value) => {
  try {
    const cipher = crypto__default["default"].createCipheriv(algorithm, key, iv);
    let res = cipher.update(JSON.stringify(value), 'utf8', 'hex');
    res += cipher.final('hex');
    return res
  } catch (error) {
    console.error(error);
    return null
  }
};

const decode = (rawValue) => {
  try {
    const decipher = crypto__default["default"].createDecipheriv(algorithm, key, iv);
    let res = decipher.update(rawValue, 'hex', 'utf8');
    res += decipher.final('utf8');
    return JSON.parse(res)
  } catch (error) {
    console.error(error);
    return null
  }
};

/**
 * @type {string[]}
 */
const privateKeys = [];
class Store {
  /**
   * 
   * @param {import('#/index').default} sdk 
   */
  constructor(sdk) {
    this.sdk = sdk;
    this.localPath = path__default["default"].join(sdk.userDataPath, 'appStore.json');
    this.sdkId = sdk.sdkId;
    this.inited = false;
    /**
     * @type {Record<string, unknown>}
     */
    this.data = {};
    this.init();
  }

  async init() {
    this._registerIpcMain();
    this._loadDataFromLocalFile();
  }

  _registerIpcMain() {
    electron.ipcMain.on(`APP_STORE_SET-${this.sdkId}`, (event, params) => {
      event.returnValue = this.set(params.key, params.value);
    });
    electron.ipcMain.on(`APP_STORE_DEL-${this.sdkId}`, (event, params) => {
      event.returnValue = this.delete(params.key);
    });
    electron.ipcMain.on(`APP_STORE_CLEAR-${this.sdkId}`, (event, params) => {
      event.returnValue = this.clear();
    });
    electron.ipcMain.on(`APP_STORE_GET-${this.sdkId}`, (event, params) => {
      event.returnValue = this.get(params.key);
    });
    electron.ipcMain.on(`APP_STORE_GET_ALL-${this.sdkId}`, (event, params) => {
      event.returnValue = this.getAll();
    });
  }

  _loadDataFromLocalFile(isFirstApp) {
    try {
      if (fs__default$1["default"].existsSync(this.localPath)) {
        const dataStr = fs__default$1["default"].readFileSync(this.localPath, { encoding: 'utf-8' });
        this.data = this.data = JSON.parse(dataStr || '{}');
      }
    } catch (error) {
      this.data = {};
      console.error('store _loadDataFromLocalFile error:', error);
    }
    this.inited = true;
  }

  _saveDataToLocalFile = debounce__default["default"](() => {
    try {
      if (!this.inited) return
      fs__default$1["default"].writeFileSync(this.localPath, JSON.stringify(this.data));
    } catch (error) {
      console.error('store _saveDataToLocalFile error:', error);
    }
  }, 1000).bind(this)

  /**
   * 
   * @param {string} key 
   * @param {unknown} value 
   * @returns 
   */
  set(key, value) {
    try {
      if (typeof key !== 'string') throw new TypeError('key must be string')
      this.data[key] = privateKeys.find((item) => key === item) ? encode(value) : value;
      this._saveDataToLocalFile();
      return value
    } catch (error) {
      console.error('store set data error:', error);
    }
  }

  /**
   * 
   * @param {string} key 
   * @returns 
   */
  delete(key) {
    try {
      if (typeof key !== 'string') throw new TypeError('key must be string')
      delete this.data[key];
      this._saveDataToLocalFile();
      return true
    } catch (error) {
      console.error('store delete data error:', error);
    }
  }

  clear() {
    try {
      this.data = {};
      this._saveDataToLocalFile();
    } catch (error) {
      console.error('store clear data error:', error);
    }
  }

  /**
   * 
   * @param {string} key 
   * @returns 
   */
  get(key) {
    try {
      if (typeof key !== 'string') throw new TypeError('key must be string')
      if (privateKeys.find((item) => key === item)) {
        return decode(this.data[key])
      }
      return this.data[key]
    } catch (error) {
      console.error('store set data error:', error);
      return null
    }
  }

  getAll() {
    const res = {};
    Object.keys(this.data).forEach(key => {
      res[key] = this.get(key);
    });
    return res
  }
}

// note preview sdk 还依赖了 remote, 这部分重构逻辑过于复杂，暂时用 @electron/remote
// 其中web的部分已经完全剥离 remote 模块
// require('@electron/remote/main').initialize()

electron.ipcMain.handle('dialog.showOpenDialog', async (event, options) => {
  const res = await electron.dialog.showOpenDialog(electron.BrowserWindow.getFocusedWindow(), options);
  return res
});

electron.ipcMain.handle('dialog.showSaveDialog', async (event, options) => {
  const res = await electron.dialog.showSaveDialog(electron.BrowserWindow.getFocusedWindow(), options);
  return res
});

electron.ipcMain.on('app.getPath', (event, name) => {
  try {
    event.returnValue = electron.app.getPath(name);
  } catch (error) {
    event.returnValue = error.toString();
  }
});

electron.ipcMain.on('app.isPackaged', (event) => {
  try {
    event.returnValue = electron.app.isPackaged;
  } catch (error) {
    event.returnValue = error.toString();
  }
});

electron.ipcMain.on('app.getAppPath', (event, name) => {
  try {
    event.returnValue = electron.app.getAppPath();
  } catch (error) {
    event.returnValue = error.toString();
  }
});

electron.ipcMain.on('app.getVersion', (event, name) => {
  try {
    event.returnValue = electron.app.getVersion();
  } catch (error) {
    event.returnValue = error.toString();
  }
});

electron.ipcMain.on('Menu.buildContextMenu', (event, items) => {
  const menu = new electron.Menu();
  items.forEach((item) => {
    if (typeof item === 'object') {
      if (item.label) {
        item.click = () => { event.sender.send('Menu.onClick', item.label); };
      }
      const menuItem = new electron.MenuItem(item);
      menu.append(menuItem);
    }
  });
  menu.items = items;
  menu.popup(electron.BrowserWindow.getFocusedWindow());
});

electron.ipcMain.on('BrowserWindow.curWindow.proxy', (event, actionKey, params) => {
  try {
    const curWindow = electron.BrowserWindow.getFocusedWindow();
    event.returnValue = curWindow[actionKey](...params);
  } catch (error) {
    event.returnValue = error.toString();
  }
});

electron.ipcMain.on('SEND_DEBUG_COMMAND', (event, command, payload) => {
  if (event.sender.debugger) {
    if (!event.sender.debugger.isAttached()) {
      event.sender.debugger.attach('1.3');
    }
    event.sender.debugger.sendCommand(command, payload);
    if (command === 'Emulation.setEmitTouchEventsForMouse') {
      electron.webContents.getAllWebContents().forEach(webcontent => {
        webcontent.send('EMULATION_STATE_CHANGE', payload);
      });
    } 
  }
});

electron.ipcMain.handle('setServiceCookie', async (event, data) => {
  try {
    const { url, cookieStr } = data;

    const cookieObj = Object.fromEntries(
      cookieStr.split('; ').map(c => {
        const [key, ...v] = c.split('=');
        return [key, v.join('=')]
      })
    );

    const serviceWebContent = event.sender;
    const cookie = serviceWebContent.session.cookies;

    await Promise.all(
      Object.keys(cookieObj).map(key => {
        return cookie.set({
          url,
          name: key,
          value: cookieObj[key].replace(';', ''),
        })
      })
    );

    return true
  } catch (error) {
    console.log(error);
    return false
  }
});

electron.ipcMain.handle('clearServiceCookie', async (event, data) => {
  try {
    const { url } = data;

    const serviceWebContent = event.sender;
    const cookie = serviceWebContent.session.cookies;

    const cookies = await cookie.get({ url });

    await Promise.all(
      cookies.map(item => {
        return cookie.remove(url, item.name)
      })
    );
    return true
  } catch (error) {
    console.log(error);
    return false
  }
});

// 读基础库信息 by dir / zip
// 下载基础库 by url （dest）
// 设置基础库 by zip

const readFrameworkConfigByDir = (dir) => {
  const configPath = path__default["default"].join(dir, 'config.json');
  if (!fs__default["default"].existsSync(configPath)) {
    throw new Error('基础库 config.json 不存在')
  }
  const configStr = fs__default["default"].readFileSync(configPath, 'utf8');
  const config = JSON.parse(configStr);
  if (!config.FrameworkVersion) {
    throw new Error('基础库 config.json 不存在')
  }
  const data = {
    iosMinimumVer: config.IOSSDKVersion,
    androidMinimumVer: config.AndroidSDKVersion,
    version: config.FrameworkVersion,
    basicVer: config.FrameworkVersion,
  };
  return data
};

let isPrepared = false;

const watchConfig = {
  usePolling: false,
  ignoreInitial: true,
  persistent: true,
  awaitWriteFinish: {
    stabilityThreshold: 1000,
    pollInterval: 100
  },
};

const DEFAULT_FIDE_PROJECT_CONFIG = {
  "description": "项目配置文件",
  "packOptions": {
    "ignore": []
  },
  "setting": {
    "useOldBuilder": false
  },
  "compileType": "miniprogram",
  "appid": "",
  "projectname": "",
  "isGameTourist": false,
  "isFulljs": false
};

class PreviewSDKMain extends emitter__default["default"] {
  constructor(sdkId) {
    super();
    /**
     * @type {Service} 
     */
    this.service = null;
    /**
     * @type {Route}
     */
    this.route;
    this.watchers = [];
    this.mainSDKPrepared = false;
    this.ready = false;
    this.serverPort = null;
    this.remoteDebugPort;
    /**
     * @type {AppStore}
     */
    this.appStore = { data: {} };
    this.sdkId = sdkId;
    this.pendingEvents = [];
    this.userDataPath;
    this.projectRoot;
    this.compileOutputPath;
  }

  initAppStore(appStore) {
    if (appStore) this.appStore = appStore;
    else {
      this.appStore = new Store(this);
    }
  }

  _emitSdkEvent(event, payload = {}) {
    if (!this.ready) {
      this.pendingEvents.push({ event, payload });
    } else {
      this.sendToHome(`sdkEvent-${this.sdkId}`, { event, payload });
    }
    this.emit(event, payload);
  }

  _handleSDKEvent(ipcEvent, data) {
    const { event, payload } = data;
    this.emit(event, payload);
  }

  async prepare(config = {}) {
    if (isPrepared) {
      this.serverPort = global.__IDE_SDK_SERVER_PORT;
    } else {
      await this._startServer(config.serverPort || 7789);
      this._registerProtocol();
    }
    isPrepared = true;
  }

  _startServer(serverPort) {
    return new Promise((resolve, reject) => {
      portfinder__default["default"].getPort({ port: serverPort }, (err, port) => {
        if (err) {
          console.error('Could not get a free port for server', err);
          reject(err);
        }

        const server = express__default["default"]();
        const rootPath = path__default["default"].join(electron.app.getPath('userData'), './preview');
        server.use('/', (req, res, next) => {
          if (req.query.path) {
            res.sendFile(req.query.path, {
              root: path__default["default"].join(rootPath, 'miniapp', this.id || this.name, 'app')
            });
          } else {
            express__default["default"].static(rootPath)(req, res, next);
          }
        });
        server.set('port', port);
        const httpServer = http__default["default"].createServer(server);
        httpServer.listen(port);
        serverPort = port;
        global.__IDE_SDK_SERVER_PORT = port;
        this.serverPort = port;

        httpServer.on('listening', () => {
          console.log('preview sdk server listening on port:', port);
          resolve(true);
        });
      });
    })
  }

  _registerProtocol() {
    electron.protocol.registerFileProtocol('localfile', (request, callback) => {
      const rootPath = path__default["default"].join(electron.app.getPath('userData'), './preview');
      let pathname;
      pathname = decodeURI(request.url.replace('localfile://', ''));
      if (/framework\/css\/index.css/.test(request.url)) pathname = path__default["default"].join(rootPath, 'framework/css/index.css');
      if (/framework\/script\/view.js/.test(request.url)) pathname = path__default["default"].join(rootPath, 'framework/script/view.js');
      callback(pathname);
    });
    electron.protocol.registerFileProtocol('finfile', (request, callback) => {
      const url = new URL(request.url);
      const l = request.url.replace(url.search, '').replace('finfile://', '');
      const pathname = path__default["default"].join(this.fileCachePath, decodeURI(l));
      callback(pathname);
    });
    electron.protocol.registerBufferProtocol('finrequest', (request, callback) => {
      const url = request.url.replace('finrequest://', '');
      let name = null;
      let mimeType = 'application/octet-stream';
      if (url.startsWith('readzipentry')) {
        name = 'readzipentry';
      }
      if (url.startsWith('cameraframe')) {
        name = 'cameraframe';
      }
      if (url.startsWith('framerecorded')) {
        name = 'framerecorded';
      }
      const options = {};
      if (name) {
        const searchParams = new URL(request.url).searchParams;
        const start = parseInt(searchParams.get('start') || '');
        let len = parseInt(searchParams.get('len') || '');
        let file = path__default["default"].join(this.fileCachePath, 'temp', name);
        fs__default["default"].ensureDirSync(path__default["default"].join(this.fileCachePath, 'temp'));
        if (name === 'readzipentry') {
          len = len || parseInt(searchParams.get('size') || '');
          options.encoding = searchParams.get('encoding');
          if (options.encoding !== 'binary') {
            mimeType = 'text/plain';
          }
          const dirType = searchParams.get('dirType');
          file = path__default["default"].join(
            this.fileCachePath,
            dirType,
            decodeURIComponent(searchParams.get('path') || '')
          );
        }
        const reader = fs__default["default"].createReadStream(file, {
          start: start,
          end: start + len,
          options,
        });
        let result = Buffer.from('');
        reader.on('data', (data) => {
          result = Buffer.concat([result, data]);
        });
        reader.on('end', () => {
          // eslint-disable-next-line
          callback({ mimeType: mimeType, data: result });
          result = null;
          reader.close();
        });
      }
    });
    electron.session.defaultSession.webRequest.onBeforeSendHeaders(
      (details, callback) => {
        if (/127.0.0.1|localhost/.test(details.requestHeaders.Referer)) {
          delete details.requestHeaders.Referer;
        }
        // eslint-disable-next-line
        callback({ requestHeaders: details.requestHeaders });
      }
    );
    this._emitSdkEvent('PROTOCOL_REGISTERED');
  }

  _startCursorPosInterval() {
    if (!this.homeWin.cursorPosIntervalId) {
      this.homeWin.cursorPosIntervalId = setInterval(() => {
        if (this.homeWin && this.homeWin.isDestroyed()) {
          clearInterval(this.homeWin.cursorPosIntervalId);
        } else {
          const contentBounds = this.homeWin.getContentBounds();
          this.homeWin.webContents.send('WINDOW_POSITION', {
            pos: [contentBounds.x, contentBounds.y],
            cursor: electron.screen.getCursorScreenPoint(),
            isFocused: this.homeWin.isFocused()
          });
        }
      }, 200);
    }
  }

  _readFrameworkInfo() {
    const frameworkPath = path__default["default"].join(this.userDataPath, 'preview', 'framework');
    try {
      const isExist = fs__default["default"].existsSync(frameworkPath);
      if (!isExist) {
        this._emitSdkEvent('LOAD_FRAMEWORK_ERROR', 'framework unexist');
      }
      const frameworkInfo = readFrameworkConfigByDir(frameworkPath);
      this.appStore.set('currentAppletVersion', frameworkInfo);
    } catch (error) {
      this._emitSdkEvent('LOAD_FRAMEWORK_ERROR', error.toString());
    }
  }

  async init(homeWin, appStore, config) {
    this.userDataPath = electron.app.getPath('userData');
    if (config.remoteDebugPort) {
      this.remoteDebugPort = config.remoteDebugPort;
    }
    this.initAppStore(appStore);
    await this.prepare();
    this._registerEventHandler();
    this.homeWin = homeWin;
    this._startCursorPosInterval();
    this._readFrameworkInfo();
    this.homeEventManager = new EventManager(
      this,
      `home-${this.sdkId}`,
      this.homeWin.webContents.id,
      'home'
    );
    this.route = new Route(this);

    if (!electron.app.isPackaged) {
      this.startframeworkWatcher();
    }
    this.mainSDKPrepared = true;
    this._emitSdkEvent('SDK_MAIN_READY', {
      serverPort: this.serverPort || global.__IDE_SDK_SERVER_PORT,
      userDataPath: electron.app.getPath('userData'),
      remoteDebugPort: this.remoteDebugPort
    });
    this.sendToHome(`mainSDKReady-${this.sdkId}`);
  }

  loadConfig(projectInfo) {
    this.name = projectInfo.name;
    this.id = projectInfo.id;
    this.path = projectInfo.path;
    this.projectRoot = projectInfo.path;
    this.appId = projectInfo.appId;
    this.devMode = projectInfo.devMode;
    this.extConfig = {}; // app.ext.json
    this.ftConfig = {}; // 对应的是 service. window.__wxConfig__ || app.json
    this.finclipConfig = {}; // 对应的是 FinClipConf.js 导出的内容
    this.projectConfig = null; // 对应 project.config.json 的项目配置
    this.appConfig = null;
    this.compileOutputPath = path__default["default"].join(this.userDataPath, `./preview/miniapp/${this.id || this.name}/__APP__`);
    this.buildOption = projectInfo.buildOption || {};

    this._loadProjectConfig();
    this._loadAppJsonConfig();
    this._loadFinClipConf();
    this._loadAppExtConfig();
    this._emitSdkEvent('CONFIG_LOADED', {
      projectInfo,
      projectConfig: this.projectConfig,
      appExtJson: this.appExtConfig,
      appJson: this.ftConfig,
      finclipConfig: this.finclipConfig,
      compileOutputPath: this.compileOutputPath,
      currentAppletVersion: this.appStore.currentAppletVersion,
      buildOption: this.buildOption
    });
  }

  start() {
    this.webviewSDK = WebviewSDK.init(this);
    this.route.reload();

    const service = new Service(this);
    this.service = service;
    this._initCache();
  }

  reloadApp() {
    this._emitSdkEvent('resetRenderState');
    this.route && this.route.reload();
    this.service && this.service.reload();
    this.webviewSDK && this.webviewSDK.reload();
  }

  async launchApp(projectInfo) {
    this.loadConfig(projectInfo);
    if (this.appStore.data.ignoreCompile) {
      this.start();
      return
    }
    this.appStore.set('mockApi', {});
    await this.buildApp(this.buildOption);
    this.start();
  }

  _loadProjectConfig() {
    let projectConfig = {};

    const setProjectConfig = (name) => {
      if (fs__default["default"].existsSync(path__default["default"].join(this.projectRoot, name))) {
        try {
          const config = fs__default["default"].readJSONSync(path__default["default"].join(this.projectRoot, name));
          projectConfig = { ...projectConfig, ...config };
        } catch (error) {}
      }
    };

    setProjectConfig('fide.project.config.json');
    setProjectConfig('project.config.json');
    setProjectConfig('project.private.config.json');

    if(!Object.keys(projectConfig)){
      projectConfig = { ...DEFAULT_FIDE_PROJECT_CONFIG };
    }

    if (projectConfig.miniprogramRoot) {
      this.path = path__default["default"].join(this.path, projectConfig.miniprogramRoot);
    }
    this.projectConfig = projectConfig;
  }

  _loadAppJsonConfig() {
    let ftConfig = {};
    try {
      ftConfig = fs__default["default"].readJSONSync(path__default["default"].join(this.path, 'app.json'));
    } catch (error) {
      ftConfig = {};
      console.error(error);
    }
    this.ftConfig = ftConfig;
  }

  _loadAppExtConfig() {
    let extConfig = {};
    try {
      extConfig = fs__default["default"].readJSONSync(path__default["default"].join(this.path, 'app.ext.json'));
    } catch (error) {
      extConfig = {};
    }
    this.appExtConfig = extConfig;
  }

  _loadFinClipConf() {
    let finclipConfig = {};
    const configPath = path__default["default"].join(this.path, 'FinClipConf.js');
    if (fs__default["default"].existsSync(configPath)) {
      try {
        delete require.cache[require.resolve(configPath)];
        finclipConfig = require(configPath);
      } catch (error) {
        finclipConfig = {};
        console.error(error);
      }
    }
    this.finclipConfig = finclipConfig;
  }

  _registerEventHandler() {
    electron.ipcMain.on(`rendererSDKCreated-${this.sdkId}`, this.handleRendererSdkCreated.bind(this));
    electron.ipcMain.on(`sdkEvent-${this.sdkId}`, this._handleSDKEvent.bind(this));
    // preload 的执行在 event 模块注册之前，所以部分事件的处理逻辑需要在此处完成
    electron.ipcMain.on(`getProjectInfo-${this.sdkId}`, (event, data) => {
      try {
        event.returnValue = {
          name: this.name,
          path: this.path,
          appId: this.appId,
          id: this.id,
          fileCachePath: this.fileCachePath,
          storagePath: this.storagePath,
          deviceInfo: this.deviceInfo,
          mapConfig: this.appStore.get('mapConfig')
        };
      } catch (error) {
        event.returnValue = error.toString();
      }
    });

    this.on('appEnterForeground', this.handleAppEnterForeground.bind(this));
    this.on('appEnterBackground', this.handleappEnterBackground.bind(this));
    this.on('openApplet', this.openApplet.bind(this));
    this.on('relaunchApplet', this.relaunchApplet.bind(this));
    this.on('reloadApplet', this.reloadApplet.bind(this));
    this.on('closeApplet', this.closeApplet.bind(this));
    this.on('CALL_BUILD_APP_FUNC', this.buildApp.bind(this));
    this.on('CLEAR_CACHE', this.clearCache.bind(this));
    this.on('setDevice', this._handleSetDevice.bind(this));
    this.on('userCaptureScreen', this._handleUserCaptureScreen.bind(this));
    this.on('pull-down-refresh', this._handlePullDownRefresh.bind(this));
  }

  openApplet(projectInfo) {
    this.stop();
    this.launchApp(projectInfo);
  }

  relaunchApplet() {
    this.route && this.route.reload();
    this.webviewSDK && this.webviewSDK.reload();
    this.service && this.service.destroy();
    this.service = null;
    this.start();
  }

  reloadApplet() {
    this.reloadApp();
  }

  closeApplet() {
    this.stop();
  }

  _handleSetDevice(deviceInfo) {
    this.deviceInfo = deviceInfo;
  }

  _handleUserCaptureScreen(){
    this.service.send("customEvent", {
      event: "onUserCaptureScreen",
      params: {},
    }); 
  }

  _handlePullDownRefresh(data = {}) {
    this.sendToService('customEvent', {
      event: 'onPullDownRefresh',
      params: { webviewId: data.webviewId, refreshType: data.refreshType },
    });
  }

  handleRendererSdkCreated() {
    this.ready = true;

    if (this.pendingEvents.length) {
      this.pendingEvents.forEach(item => {
        this.sendToHome(`sdkEvent-${this.sdkId}`, { event: item.event, payload: item.payload });
      });
    }
  }

  handleAppEnterForeground() {
    this.service.send('customEvent', {
      event: 'onAppEnterForeground',
      params: {
        path: this.route.currentView.path.replace('.html', ''),
        query: this.route.currentView.query,
        scene: 1001,
        referrerInfo: {}
      }
    });
  }

  handleappEnterBackground() {
    this.service.send('customEvent', {
      event: 'onAppEnterBackground',
      params: { mode: 'hide' }
    });
  }

  async setServiceCookie(event, data) {
    try {
      const { url, cookieStr } = data;

      const cookieObj = Object.fromEntries(
        cookieStr.split('; ').map(c => {
          const [key, ...v] = c.split('=');
          return [key, v.join('=')]
        })
      );

      const serviceWebContent = electron.webContents.fromId(this.service.event.id);
      const cookie = serviceWebContent.session.cookies;

      await Promise.all(
        Object.keys(cookieObj).map(key => {
          return cookie.set({
            url,
            name: key,
            value: cookieObj[key].replace(';', ''),
          })
        })
      );

      return true
    } catch (error) {
      console.log(error);
      return false
    }
  }

  async clearServiceCookie(event, data) {
    try {
      const { url } = data;

      const serviceWebContent = electron.webContents.fromId(this.service.event.id);
      const cookie = serviceWebContent.session.cookies;

      const cookies = await cookie.get({ url });

      await Promise.all(
        cookies.map(item => {
          return cookie.remove(url, item.name)
        })
      );
      return true
    } catch (error) {
      console.log(error);
      return false
    }
  }

  _initCache() {
    this.fileCachePath = path__default["default"].join(
      electron.app.getPath('userData'),
      'preview',
      'filecache',
      this.id || this.name
    );
    this.storagePath = path__default["default"].join(
      electron.app.getPath('userData'),
      'preview',
      'storage',
      this.id || this.name
    );
    fs__default["default"].ensureDir(path__default["default"].join(this.fileCachePath, 'temp'));
    fs__default["default"].ensureDir(path__default["default"].join(this.fileCachePath, 'store'));
    fs__default["default"].ensureDir(path__default["default"].join(this.fileCachePath, 'usr'));
    fs__default["default"].ensureDir(this.storagePath);
  }

  clearCache(type = 'all') {
    if (type === 'file') this.clearFileCache();
    if (type === 'storage') this.clearStorageCache();
    if (type === 'compile') this.clearCompileCache();
    if (type === 'all') {
      this.clearFileCache();
      this.clearStorageCache();
      this.clearCompileCache();
    }
  }

  clearStorageCache() {
    fs__default["default"].removeSync(this.storagePath);
    fs__default["default"].ensureDir(this.storagePath);

    this.sendToService('syncStorageAll');
    this._emitSdkEvent('CLEAR_CACHE_DONE', {
      type: 'storage'
    });
  }

  clearFileCache() {
    const tempPath = path__default["default"].join(this.fileCachePath, 'temp');
    const storePath = path__default["default"].join(this.fileCachePath, 'store');
    fs__default["default"].removeSync(tempPath);
    fs__default["default"].ensureDir(tempPath);
    fs__default["default"].removeSync(storePath);
    fs__default["default"].ensureDir(storePath);

    this._emitSdkEvent('CLEAR_CACHE_DONE', {
      type: 'file'
    });
  }

  async clearCompileCache() {
    this.stop();

    const destPath = path__default["default"].join(
      this.userDataPath,
      `./preview/miniapp/${this.id || this.name}`
    );
    fs__default["default"].removeSync(destPath);

    await this.buildApp(this.buildOption);
    this.relaunchApplet();

    this._emitSdkEvent('CLEAR_CACHE_DONE', {
      type: 'compile'
    });
  }

  checkWindowFocus(event, params) {
    const sender = event.sender;
    const webContent = electron.webContents.fromId(sender.id);
    const browserWindow = electron.BrowserWindow.fromWebContents(webContent);
    browserWindow.on('focus', () => {
      sender.send('WINDOW_FOCUS', { focus: true });
    });
    browserWindow.on('blur', () => {
      sender.send('WINDOW_FOCUS', { focus: false });
    });
  }

  getWindowPosition(event, params) {
    const sender = event.sender;
    const webContent = electron.webContents.fromId(sender.id);
    const browserWindow = electron.BrowserWindow.fromWebContents(webContent);
    const pos = browserWindow.getPosition();
    const cursor = electron.screen.getCursorScreenPoint();
    event.returnValue = {
      pos,
      cursor,
    };
  }

  stop() {
    this._emitSdkEvent('resetRenderState');
    this.route && this.route.reload();
    this.webviewSDK && this.webviewSDK.reload();
    this.service && this.service.destroy();
    this.service = null;
    builderV2.stop();
  }

  send(event, data, webContentId) {
    if (webContentId) {
      const webContent = electron.webContents.fromId(webContentId);
      webContent && webContent.send(event, data);
    }
  }

  sendToService(event, data) {
    this.service.send(event, data);
  }

  sendToViews(event, params, webviewIds) {
    let views = this.route.stack.filter(item =>
      webviewIds.includes(`${item.webviewId}`)
    );
    if (views.length === 0) {
      views = this.route.tabs.filter(item =>
        webviewIds.includes(`${item.webviewId}`)
      );
    }
    views.forEach(item => {
      item.send('customEvent', {
        event,
        params,
      });
    });
  }

  sendToHome(event, data) {
    if (this.homeWin && !this.homeWin.isDestroyed()) {
      this.homeWin.webContents.send(event, data);
    }
  }

  async execBuildApp(event, dir, destPath, option, useExtraProcess) {
    const buildOption = { ...option };
    buildOption.plugins = {};
    if (this.ftConfig.plugins) {
      Object.assign(buildOption.plugins, this.ftConfig.plugins);
    }

    if (
      (this.ftConfig.subPackages && this.ftConfig.subPackages.length) ||
      (this.ftConfig.subpackages && this.ftConfig.subpackages.length)
    ) {
(this.ftConfig.subPackages || this.ftConfig.subpackages).forEach(
        item => {
          if (item.plugins) {
            Object.assign(buildOption.plugins, item.plugins);
          }
        }
      );
    }
    if (this.projectConfig.pluginRoot) {
      buildOption.pluginRoot = path__default["default"].join(
        this.projectRoot,
        this.projectConfig.pluginRoot
      );
    }

    buildOption.isFulljs = this.projectConfig.isFulljs || false;
    buildOption.theme = 'light';
    buildOption.pluginExportId = this.projectConfig.projectname + '_plugin';
    await builderV2.start(event, this.devMode, dir, destPath, buildOption, useExtraProcess);
  }

  /**
   * 编译小程序
   * @param { Object } event ipcMain提供的事件对象
   * @param {Boolean} watch 是否监听小程序代码变动
   * @returns
   */
  async buildApp(inputOptions = this.buildOption) {
    const options = Object.assign({}, this.buildOption, inputOptions);

    console.log('buildApp', options);

    const srcPath = options.srcPath || this.path;
    const destPath = options.destPath || path__default["default"].join(
      this.userDataPath,
      `./preview/miniapp/${this.id || this.name}`
    );
    let msgChannel;
    if (options.msgChannel) {
      msgChannel = options.msgChannel;
    }

    const skipCompileWatch = typeof options.skipCompileWatch !== 'undefined' ? options.skipCompileWatch : this.appStore.data.skipCompileWatch;
    const event = {};
    event.sender = {};
    event.sender.send = (key, value) => {
      this._emitSdkEvent('BUILD', {
        type: key,
        value,
        channel: msgChannel
      });
    };

    console.log('require.resolve @finclip/applet-builder start', Date.now());
    const option = {
      zip: true,
      onWatcherBuild: () => {
        this.relaunchApplet();
      },
      reloadBuild: async () => {
        await this.buildApp(this.buildOption);
        this._emitSdkEvent('resetRenderState');
        this.relaunchApplet();
      },
      watch: !skipCompileWatch,
      ...options,
      builderAbsolutePath: require.resolve('@finclip/applet-builder')
    };
    console.log('require.resolve @finclip/applet-builder end', Date.now());

    // destPath 的时候另起一个线程来进行编译
    const useExtraProcess = !!options.destPath;
    await this.execBuildApp(event, srcPath, destPath, option, useExtraProcess);
  }

  // 对基础库文件进行监听（非生产环境使用）
  startframeworkWatcher() {
    const userDataPath = electron.app.getPath('userData');
    const watcher = chokidar__default["default"].watch(
      path__default["default"].join(userDataPath, './preview/dev-framework'),
      watchConfig
    );
    watcher.on('all', async (e, dir) => {
      fs__default["default"].removeSync(path__default["default"].join(userDataPath, './preview/framework'));
      fs__default["default"].copySync(
        path__default["default"].join(userDataPath, './preview/dev-framework'),
        path__default["default"].join(userDataPath, './preview/framework')
      );
      this.reloadApp();
      this._emitSdkEvent('resetRenderState');
    });
  }

  registerExtApiModule(modulePath) {
    this.service.send('registerExtApiModule', modulePath);
  }

  getCompileOutputDir() {
    return path__default["default"].join(this.compileOutputPath, '../')
  }
}

module.exports = PreviewSDKMain;
